#-------------------------------------------------------------------------------
# preprocessor.py
# 
# Verilog Preprocessor
# 
# Icarus Verilog is used as a preprocessor via command-line.
# Please install Icarus Verilog on your environment.
#
# Copyright (C) 2013, Shinya Takamaeda-Yamazaki
# License: Apache 2.0
#-------------------------------------------------------------------------------

import sys
import os
import subprocess
import re

sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))) )

class VerilogPreprocessor(object):
    def __init__(self, filelist, outputfile='pp.out', include=None, define=None):
        self.filelist = filelist
        cmd = []
        cmd.append('iverilog ')
        if include:
            for inc in include:
                cmd.append('-I ')
                cmd.append(inc)
                cmd.append(' ')
        if define:
            for d in define:
                cmd.append('-D')
                cmd.append(d)
                cmd.append(' ')
        cmd.append('-E -o ')
        cmd.append(outputfile)
        self.iv = ''.join(cmd)
    def preprocess(self):
        cmd = self.iv + ' '
        for f in self.filelist:
            cmd += ' ' + f
        subprocess.call(cmd, shell=True)

#-------------------------------------------------------------------------------        
def preprocess(filelist,
               output='preprocess.output', include=None, define=None):
    pre = VerilogPreprocessor(filelist, output, include, define)
    pre.preprocess()
    text = open(output).read()
    os.remove(output)
    return text

#-------------------------------------------------------------------------------
if __name__ == '__main__':
    import pyverilog.utils.version
    from optparse import OptionParser

    INFO = "Verilog Preprocessor"
    VERSION = pyverilog.utils.version.VERSION
    USAGE = "Usage: python preprocessor.py file ..."

    def showVersion():
        print(INFO)
        print(VERSION)
        print(USAGE)
        sys.exit()

    optparser = OptionParser()
    optparser.add_option("-v","--version",action="store_true",dest="showversion",
                         default=False,help="Show the version")
    optparser.add_option("-I","--include",dest="include",action="append",
                         default=[],help="Include path")
    optparser.add_option("-D",dest="define",action="append",
                         default=[],help="Macro Definition")
    (options, args) = optparser.parse_args()

    filelist = args
    if options.showversion:
        showVersion()

    for f in filelist:
        if not os.path.exists(f): raise IOError("file not found: " + f)

    if len(filelist) == 0:
        showVersion()
        
    text = preprocess(filelist, include=options.include, define=options.define)
    
    print(text)



#-------------------------------------------------------------------------------
# ast.py
#
# Verilog HDL AST classes with Pyverilog
#
# Copyright (C) 2013, Shinya Takamaeda-Yamazaki
# edited by ryosuke fukatani
# License: Apache 2.0
#-------------------------------------------------------------------------------

import sys
import re

class Node(object):
    '''Abstact class for every element in parser'''
    coord = False
    def children(self):
        pass
    def show(self, buf=sys.stdout, offset=0, attrnames=False, showcoord=False):
        indent = 2
        lead = ' ' * offset
        buf.write(lead + self.__class__.__name__ + ': ')
        if self.attr_names:
            if attrnames:
                nvlist = [(n, getattr(self,n)) for n in self.attr_names]
                attrstr = ', '.join('%s=%s' & nv for nv in nvlist)
            else:
                vlist = [getattr(self,n) for n in self.attr_names]
                attrstr = ', '.join('%s' % v for v in vlist)
            buf.write(attrstr)
        if showcoord:
            buf.write(' (at %s)' % self.coord)
        buf.write('\n')
        for c in self.children():
            c.show(buf, offset + indent, attrnames, showcoord)
    def __eq__(self, other):
        if type(self) != type(other): return False
        self_attrs = tuple( [ getattr(self, a) for a in self.attr_names ] )
        other_attrs = tuple( [ getattr(other, a) for a in other.attr_names ] )
        if self_attrs != other_attrs: return False
        other_children = other.children()
        for i, c in enumerate(self.children()):
            if c != other_children[i]: return False
        return True
    def __ne__(self, other):
        return not self.__eq__(other)
    def __hash__(self):
        s = hash(tuple([getattr(self, a) for a in self.attr_names]))
        c = hash(self.children())
        return hash((s, c))

################################################################################
class Source(Node):
    attr_names = ('name',)
    def __init__(self, name, description):
        self.name = name
        self.description = description
    def children(self):
        nodelist = []
        if self.description: nodelist.append(self.description)
        return tuple(nodelist)

class Description(Node):
    attr_names = ()
    def __init__(self, definitions):
        self.definitions = definitions
    def children(self):
        nodelist = []
        if self.definitions: nodelist.extend(self.definitions)
        return tuple(nodelist)

class ModuleDef(Node):
    attr_names = ('name',)
    def __init__(self, name, paramlist, portlist, items, default_nettype='wire'):
        self.name = name
        self.paramlist = paramlist
        self.portlist = portlist
        self.items = items
        self.default_nettype = default_nettype
    def children(self):
        nodelist = []
        if self.paramlist: nodelist.append(self.paramlist)
        if self.portlist: nodelist.append(self.portlist)
        if self.items: nodelist.extend(self.items)
        return tuple(nodelist)

class Paramlist(Node):
    attr_names = ()
    def __init__(self, params):
        self.params = params
    def children(self):
        nodelist = []
        if self.params: nodelist.extend(self.params)
        return tuple(nodelist)

class Portlist(Node):
    attr_names = ()
    def __init__(self, ports):
        self.ports = ports
    def children(self):
        nodelist = []
        if self.ports: nodelist.extend(self.ports)
        return tuple(nodelist)

class Port(Node):
    attr_names = ('name','type',)
    def __init__(self, name, width, type):
        self.name = name
        self.width = width
        self.type = type
    def children(self):
        nodelist = []
        if self.width: nodelist.append(self.width)
        return tuple(nodelist)

class Width(Node):
    attr_names = ()
    def __init__(self, msb, lsb):
        self.msb = msb
        self.lsb = lsb
    def children(self):
        nodelist = []
        if self.msb: nodelist.append(self.msb)
        if self.lsb: nodelist.append(self.lsb)
        return tuple(nodelist)
class Length(Width): pass

class Identifier(Node):
    attr_names = ('name',)
    def __init__(self, name, scope=None):
        self.name = name
        self.scope = scope
    def children(self):
        nodelist = []
        if self.scope: nodelist.append(self.scope)
        return tuple(nodelist)
    def __repr__(self):
        if self.scope is None:
            return self.name
        return self.scope.__repr__() + '.' + self.name

class Value(Node):
    attr_names = ()
    def __init__(self, value):
        self.value = value
    def children(self):
        nodelist = []
        if self.value: nodelist.append(self.value)
        return tuple(nodelist)

class Constant(Value):
    attr_names = ('value',)
    def __init__(self, value):
        self.value = value
    def children(self):
        nodelist = []
        return tuple(nodelist)
    def __repr__(self):
        return str(self.value)

class IntConst(Constant): pass
class FloatConst(Constant): pass
class StringConst(Constant): pass

class Variable(Value):
    attr_names = ('name', 'signed')
    def __init__(self, name, width=None, signed=False):
        self.name = name
        self.width = width
        self.signed = signed
    def children(self):
        nodelist = []
        if self.width: nodelist.append(self.width)
        return tuple(nodelist)

class Input(Variable): pass
class Output(Variable): pass
class Inout(Variable): pass
class Tri(Variable): pass
class Wire(Variable): pass
class Reg(Variable): pass
class WireArray(Variable):
    attr_names = ('name', 'signed')
    def __init__(self, name, width, length, signed=False):
        self.name = name
        self.width = width
        self.length = length
        self.signed = signed
    def children(self):
        nodelist = []
        if self.width: nodelist.append(self.width)
        if self.length: nodelist.append(self.length)
        return tuple(nodelist)
class RegArray(Variable):
    attr_names = ('name', 'signed')
    def __init__(self, name, width, length, signed=False):
        self.name = name
        self.width = width
        self.length = length
        self.signed = signed
    def children(self):
        nodelist = []
        if self.width: nodelist.append(self.width)
        if self.length: nodelist.append(self.length)
        return tuple(nodelist)
class Integer(Variable): pass
class Real(Variable): pass
class Genvar(Variable): pass

class Ioport(Node):
    attr_names = ()
    def __init__(self, first, second=None):
        self.first = first
        self.second = second
    def children(self):
        nodelist = []
        if self.first: nodelist.append(self.first)
        if self.second: nodelist.append(self.second)
        return tuple(nodelist)

class Parameter(Node):
    attr_names = ('name', 'signed')
    def __init__(self, name, value, width=None, signed=False):
        self.name = name
        self.value = value
        self.width = width
        self.signed = signed
    def children(self):
        nodelist = []
        if self.value: nodelist.append(self.value)
        if self.width: nodelist.append(self.width)
        return tuple(nodelist)
class Localparam(Parameter): pass
class Supply(Parameter) : pass

class Decl(Node):
    attr_names = ()
    def __init__(self, list):
        self.list = list
    def children(self):
        nodelist = []
        if self.list: nodelist.extend(self.list)
        return tuple(nodelist)

class Concat(Node):
    attr_names = ()
    def __init__(self, list):
        self.list = list
    def children(self):
        nodelist = []
        if self.list: nodelist.extend(self.list)
        return tuple(nodelist)
class LConcat(Concat): pass

class Repeat(Node):
    attr_names = ()
    def __init__(self, value, times):
        self.value = value
        self.times = times
    def children(self):
        nodelist = []
        if self.value: nodelist.append(self.value)
        if self.times: nodelist.append(self.times)
        return tuple(nodelist)

class Partselect(Node):
    attr_names = ()
    def __init__(self, var, msb, lsb):
        self.var = var
        self.msb = msb
        self.lsb = lsb
    def children(self):
        nodelist = []
        if self.var: nodelist.append(self.var)
        if self.msb: nodelist.append(self.msb)
        if self.lsb: nodelist.append(self.lsb)
        return tuple(nodelist)

class Pointer(Node):
    attr_names = ()
    def __init__(self, var, ptr):
        self.var = var
        self.ptr = ptr
    def children(self):
        nodelist = []
        if self.var: nodelist.append(self.var)
        if self.ptr: nodelist.append(self.ptr)
        return tuple(nodelist)

class Lvalue(Node):
    attr_names = ()
    def __init__(self, var):
        self.var = var
    def children(self):
        nodelist = []
        if self.var: nodelist.append(self.var)
        return tuple(nodelist)

class Rvalue(Node):
    attr_names = ()
    def __init__(self, var):
        self.var = var
    def children(self):
        nodelist = []
        if self.var: nodelist.append(self.var)
        return tuple(nodelist)

################################################################################
class Operator(Node):
    attr_names = ()
    def __init__(self, left, right):
        self.left = left
        self.right = right
    def children(self):
        nodelist = []
        if self.left: nodelist.append(self.left)
        if self.right: nodelist.append(self.right)
        return tuple(nodelist)
    def __repr__(self):
        ret = '('+self.__class__.__name__
        for c in self.children(): ret += ' ' + c.__repr__()
        ret += ')'
        return ret

class UnaryOperator(Operator):
    attr_names = ()
    def __init__(self, right):
        self.right = right
    def children(self):
        nodelist = []
        if self.right: nodelist.append(self.right)
        return tuple(nodelist)

################################################################################
# Level 1 (Highest Priority)
class Uplus(UnaryOperator): pass
class Uminus(UnaryOperator): pass
class Ulnot(UnaryOperator): pass
class Unot(UnaryOperator): pass
class Uand(UnaryOperator): pass
class Unand(UnaryOperator): pass
class Uor(UnaryOperator): pass
class Unor(UnaryOperator): pass
class Uxor(UnaryOperator): pass
class Uxnor(UnaryOperator): pass
################################################################################
# Level 2
class Power(Operator): pass
class Times(Operator): pass
class Divide(Operator): pass
class Mod(Operator): pass
################################################################################
# Level 3
class Plus(Operator): pass
class Minus(Operator): pass
################################################################################
# Level 4
class Sll(Operator): pass
class Srl(Operator): pass
class Sra(Operator): pass
################################################################################
# Level 5
class LessThan(Operator): pass
class GreaterThan(Operator): pass
class LessEq(Operator): pass
class GreaterEq(Operator): pass
################################################################################
# Level 6
class Eq(Operator): pass
class NotEq(Operator): pass
class Eql(Operator): pass # ===
class NotEql(Operator): pass # !==
################################################################################
# Level 7
class And(Operator): pass
class Xor(Operator): pass
class Xnor(Operator): pass
################################################################################
# Level 8
class Or(Operator): pass
################################################################################
# Level 9
class Land(Operator): pass
################################################################################
# Level 10
class Lor(Operator): pass
################################################################################
# Level 11
class Cond(Operator):
    attr_names = ()
    def __init__(self, cond, true_value, false_value):
        self.cond = cond
        self.true_value = true_value
        self.false_value = false_value
    def children(self):
        nodelist = []
        if self.cond: nodelist.append(self.cond)
        if self.true_value: nodelist.append(self.true_value)
        if self.false_value: nodelist.append(self.false_value)
        return tuple(nodelist)

################################################################################
class Assign(Node):
    attr_names = ()
    def __init__(self, left, right, ldelay=None, rdelay=None):
        self.left = left
        self.right = right
        self.ldelay = ldelay
        self.rdelay = rdelay
    def children(self):
        nodelist = []
        if self.left: nodelist.append(self.left)
        if self.right: nodelist.append(self.right)
        if self.ldelay: nodelist.append(self.ldelay)
        if self.rdelay: nodelist.append(self.rdelay)
        return tuple(nodelist)

class Always(Node):
    attr_names = ()
    def __init__(self, sens_list, statement):
        self.sens_list = sens_list
        self.statement = statement
    def children(self):
        nodelist = []
        if self.sens_list: nodelist.append(self.sens_list)
        if self.statement: nodelist.append(self.statement)
        return tuple(nodelist)

class SensList(Node):
    attr_names = ()
    def __init__(self, list):
        self.list = list
    def children(self):
        nodelist = []
        if self.list: nodelist.extend(self.list)
        return tuple(nodelist)

class Sens(Node):
    attr_names = ('type',)
    def __init__(self, sig, type='posedge'):
        self.sig = sig
        self.type = type # 'posedge', 'negedge', 'level', 'all' (*)
    def children(self):
        nodelist = []
        if self.sig: nodelist.append(self.sig)
        return tuple(nodelist)

class Substitution(Node):
    attr_names = ()
    def __init__(self, left, right, ldelay=None, rdelay=None):
        self.left = left
        self.right = right
        self.ldelay = ldelay
        self.rdelay = rdelay
    def children(self):
        nodelist = []
        if self.left: nodelist.append(self.left)
        if self.right: nodelist.append(self.right)
        if self.ldelay: nodelist.append(self.ldelay)
        if self.rdelay: nodelist.append(self.rdelay)
        return tuple(nodelist)
class BlockingSubstitution(Substitution): pass
class NonblockingSubstitution(Substitution): pass

class IfStatement(Node):
    attr_names = ()
    def __init__(self, cond, true_statement, false_statement):
        self.cond = cond
        self.true_statement = true_statement
        self.false_statement = false_statement
    def children(self):
        nodelist = []
        if self.cond: nodelist.append(self.cond)
        if self.true_statement: nodelist.append(self.true_statement)
        if self.false_statement: nodelist.append(self.false_statement)
        return tuple(nodelist)

class ForStatement(Node):
    attr_names = ()
    def __init__(self, pre, cond, post, statement):
        self.pre = pre
        self.cond = cond
        self.post = post
        self.statement = statement
    def children(self):
        nodelist = []
        if self.pre: nodelist.append(self.pre)
        if self.cond: nodelist.append(self.cond)
        if self.post: nodelist.append(self.post)
        if self.statement: nodelist.append(self.statement)
        return tuple(nodelist)

class WhileStatement(Node):
    attr_names = ()
    def __init__(self, cond, statement):
        self.cond = cond
        self.statement = statement
    def children(self):
        nodelist = []
        if self.cond: nodelist.append(self.cond)
        if self.statement: nodelist.append(self.statement)
        return tuple(nodelist)

class CaseStatement(Node):
    attr_names = ()
    def __init__(self, comp, caselist):
        self.comp = comp
        self.caselist = caselist
    def children(self):
        nodelist = []
        if self.comp: nodelist.append(self.comp)
        if self.caselist: nodelist.extend(self.caselist)
        return tuple(nodelist)

class CasexStatement(CaseStatement): pass

class Case(Node):
    attr_names = ()
    def __init__(self, cond, statement):
        self.cond = cond
        self.statement = statement
    def children(self):
        nodelist = []
        if self.cond: nodelist.extend(self.cond)
        if self.statement: nodelist.append(self.statement)
        return tuple(nodelist)

class Block(Node):
    attr_names = ('scope',)
    def __init__(self, statements, scope=None):
        self.statements = statements
        self.scope = scope
    def children(self):
        nodelist = []
        if self.statements: nodelist.extend(self.statements)
        return tuple(nodelist)

class Initial(Node):
    attr_names = ()
    def __init__(self, statement):
        self.statement = statement
    def children(self):
        nodelist = []
        if self.statement: nodelist.append(self.statement)
        return tuple(nodelist)

class EventStatement(Node):
    attr_names = ()
    def __init__(self, senslist):
        self.senslist = senslist
    def children(self):
        nodelist = []
        if self.senslist: nodelist.append(self.senslist)
        return tuple(nodelist)

class WaitStatement(Node):
    attr_names = ()
    def __init__(self, cond, statement):
        self.cond = cond
        self.statement = statement
    def children(self):
        nodelist = []
        if self.cond: nodelist.append(self.cond)
        if self.statement: nodelist.append(self.statement)
        return tuple(nodelist)

class ForeverStatement(Node):
    attr_names = ()
    def __init__(self, statement):
        self.statement = statement
    def children(self):
        nodelist = []
        if self.statement: nodelist.append(self.statement)
        return tuple(nodelist)

class DelayStatement(Node):
    attr_names = ()
    def __init__(self, delay):
        self.delay = delay
    def children(self):
        nodelist = []
        if self.delay: nodelist.append(self.delay)
        return tuple(nodelist)

class InstanceList(Node):
    attr_names = ('module',)
    def __init__(self, module, parameterlist, instances):
        self.module = module
        self.parameterlist = parameterlist
        self.instances = instances
    def children(self):
        nodelist = []
        if self.parameterlist: nodelist.extend(self.parameterlist)
        if self.instances: nodelist.extend(self.instances)
        return tuple(nodelist)

class Instance(Node):
    attr_names = ('name', 'module')
    def __init__(self, module, name, portlist, parameterlist, array=None):
        self.module = module
        self.name = name
        self.portlist = portlist
        self.parameterlist = parameterlist
        self.array = array
    def children(self):
        nodelist = []
        if self.array: nodelist.append(self.array)
        if self.parameterlist: nodelist.extend(self.parameterlist)
        if self.portlist: nodelist.extend(self.portlist)
        return tuple(nodelist)

class ParamArg(Node):
    attr_names = ('paramname',)
    def __init__(self, paramname, argname):
        self.paramname = paramname
        self.argname = argname
    def children(self):
        nodelist = []
        if self.argname: nodelist.append(self.argname)
        return tuple(nodelist)

class PortArg(Node):
    attr_names = ('portname',)
    def __init__(self, portname, argname):
        self.portname = portname
        self.argname = argname
    def children(self):
        nodelist = []
        if self.argname: nodelist.append(self.argname)
        return tuple(nodelist)

class Function(Node):
    attr_names = ('name',)
    def __init__(self, name, retwidth, statement):
        self.name = name
        self.retwidth = retwidth
        self.statement = statement
    def children(self):
        nodelist = []
        if self.retwidth: nodelist.append(self.retwidth)
        if self.statement: nodelist.extend(self.statement)
        return tuple(nodelist)
    def __repr__(self):
        return self.name.__repr__()

class FunctionCall(Node):
    attr_names = ()
    def __init__(self, name, args):
        self.name = name
        self.args = args
    def children(self):
        nodelist = []
        if self.name: nodelist.append(self.name)
        if self.args: nodelist.extend(self.args)
        return tuple(nodelist)
    def __repr__(self):
        return self.name.__repr__()

class Task(Node):
    attr_names = ('name',)
    def __init__(self, name, statement):
        self.name = name
        self.statement = statement
    def children(self):
        nodelist = []
        if self.statement: nodelist.extend(self.statement)
        return tuple(nodelist)

class TaskCall(Node):
    attr_names = ()
    def __init__(self, name, args):
        self.name = name
        self.args = args
    def children(self):
        nodelist = []
        if self.name: nodelist.append(self.name)
        if self.args: nodelist.extend(self.args)
        return tuple(nodelist)

class GenerateStatement(Node):
    attr_names = ()
    def __init__(self, items):
        self.items = items
    def children(self):
        nodelist = []
        if self.items: nodelist.extend(self.items)
        return tuple(nodelist)

class SystemCall(Node):
    attr_names = ('syscall',)
    def __init__(self, syscall, args):
        self.syscall = syscall
        self.args = args
    def children(self):
        nodelist = []
        if self.args: nodelist.extend(self.args)
        return tuple(nodelist)
    def __repr__(self):
        ret = []
        ret.append('(')
        ret.append('$')
        ret.append(self.syscall)
        for a in self.args:
            ret.append(' ')
            ret.append(str(a))
        ret.append(')')
        return ''.join(ret)

class IdentifierScopeLabel(Node):
    attr_names = ('name', 'loop')
    def __init__(self, name, loop=None):
        self.name = name
        self.loop = loop
    def children(self):
        nodelist = []
        return tuple(nodelist)

class IdentifierScope(Node):
    attr_names = ()
    def __init__(self, labellist):
        self.labellist = labellist
    def children(self):
        nodelist = []
        if self.labellist: nodelist.extend(self.labellist)
        return tuple(nodelist)

class Pragma(Node):
    attr_names = ()
    def __init__(self, entry):
        self.entry = entry
    def children(self):
        nodelist = []
        if self.entry: nodelist.append(self.entry)
        return tuple(nodelist)

class PragmaEntry(Node):
    attr_names = ('name', )
    def __init__(self, name, value=None):
        self.name = name
        self.value = value
    def children(self):
        nodelist = []
        if self.value: nodelist.append(self.value)
        return tuple(nodelist)

class Disable(Node):
    attr_names = ('dest',)
    def __init__(self, dest):
        self.dest = dest
    def children(self):
        nodelist = []
        return tuple(nodelist)

class ParallelBlock(Node):
    attr_names = ('scope',)
    def __init__(self, statements, scope=None):
        self.statements = statements
        self.scope = scope
    def children(self):
        nodelist = []
        if self.statements: nodelist.extend(self.statements)
        return tuple(nodelist)

class SingleStatement(Node):
    attr_names = ()
    def __init__(self, statement):
        self.statement = statement
    def children(self):
        nodelist = []
        if self.statement: nodelist.append(self.statement)
        return tuple(nodelist)




#-------------------------------------------------------------------------------
# bindvisitor.py
#
# Binding visitor
#
# Copyright (C) 2013, Shinya Takamaeda-Yamazaki
# edited by ryosuke fukatani
# License: Apache 2.0
#-------------------------------------------------------------------------------

import sys
import os
import re

sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))) )

from pyverilog.vparser.ast import *
import pyverilog.utils.util as util
import pyverilog.utils.verror as verror
import pyverilog.utils.signaltype as signaltype
from pyverilog.utils.scope import ScopeLabel, ScopeChain

if sys.version_info[0] >= 3:
    from pyverilog.dataflow.dataflow import *
    from pyverilog.dataflow.visit import *
    from pyverilog.dataflow.optimizer import VerilogOptimizer
    import pyverilog.dataflow.reorder as reorder
    import pyverilog.dataflow.replace as replace
else:
    from dataflow import *
    from visit import *
    from optimizer import VerilogOptimizer
    import reorder
    import replace

class BindVisitor(NodeVisitor):
    def __init__(self, moduleinfotable, top, frames, noreorder=False):
        self.moduleinfotable = moduleinfotable
        self.top = top
        self.frames = frames
        self.labels = Labels()
        self.dataflow = DataFlow()
        self.optimizer = VerilogOptimizer({}, {})

        self.noreorder = noreorder

        # set the top frame of top module
        self.frames.setCurrent(ScopeChain())
        self.stackInstanceFrame(top, top)

        self.copyAllFrameInfo()

        current = self.frames.getCurrent()
        self.copyFrameInfo(current)

        self.renamecnt = 0
        self.default_nettype = 'wire'

    ############################################################################
    def getDataflows(self):
        return self.dataflow

    def getFrameTable(self):
        return self.frames

    ############################################################################
    def start_visit(self):
        return self.visit(self.moduleinfotable.getDefinition(self.top))

    def visit_ModuleDef(self, node):
        self.default_nettype = node.default_nettype
        self.generic_visit(node)

    def visit_Input(self, node):
        self.addTerm(node)

    def visit_Output(self, node):
        self.addTerm(node)

    def visit_Inout(self, node):
        self.addTerm(node)

    def visit_Reg(self, node):
        self.addTerm(node)

    def visit_Wire(self, node):
        self.addTerm(node)

    def visit_RegArray(self, node):
        self.addTerm(node)

    def visit_WireArray(self, node):
        self.addTerm(node)

    def visit_Tri(self, node):
        self.addTerm(node)

    def visit_Integer(self, node):
        self.addTerm(node)

    def visit_Parameter(self, node):
        self.addTerm(node)
        current = self.frames.getCurrent()
        name = current + ScopeLabel(node.name, 'signal')
        if not self.hasConstant(name):
            value = self.optimize(self.getTree(node.value, current))
            self.setConstant(name, value)

        if len(self.dataflow.getBindlist(name)) == 0:
            self.addBind(node.name, node.value, bindtype='parameter')

    def visit_Supply(self, node):
        self.addTerm(node)
        current = self.frames.getCurrent()
        name = current + ScopeLabel(node.name, 'signal')
        self.addBind(node.name, node.value, bindtype='parameter')

    def visit_Localparam(self, node):
        self.addTerm(node)
        current = self.frames.getCurrent()
        name = current + ScopeLabel(node.name, 'signal')
        if not self.hasConstant(name):
            value = self.optimize(self.getTree(node.value, current))
            self.setConstant(name, value)

        self.addBind(node.name, node.value, bindtype='localparam')

    def visit_Genvar(self, node):
        self.addTerm(node)
        current = self.frames.getCurrent()
        name = self.frames.getCurrent() + ScopeLabel(node.name, 'signal')
        value = DFEvalValue(0)
        self.setConstant(name, value)

    def visit_Function(self, node):
        self.frames.setFunctionDef()
        self.generic_visit(node)
        self.frames.unsetFunctionDef()

    def visit_Task(self, node):
        self.frames.setTaskDef()
        self.generic_visit(node)
        self.frames.unsetTaskDef()

    def visit_InstanceList(self, node):
        for i in node.instances:
            self.visit(i)

    def visit_Instance(self, node):
        if node.array: return self._visit_Instance_array(node)
        nodename = node.name
        return self._visit_Instance_body(node, nodename)

    def _visit_Instance_array(self, node):
        if node.name == '':
            raise verror.FormatError("Module %s requires an instance name" % node.module)

        current = self.frames.getCurrent()
        msb = self.optimize(self.getTree(node.array.msb, current)).value
        lsb = self.optimize(self.getTree(node.array.lsb, current)).value
        num_of_pins = msb + 1 - lsb

        for i in range(lsb, msb+1):
            nodename = node.name + '_' + str(i)
            self._visit_Instance_body(node, nodename, arrayindex=i)

    def _visit_Instance_body(self, node, nodename, arrayindex=None):
        if node.module in primitives: return self._visit_Instance_primitive(node, arrayindex)

        if nodename == '':
            raise verror.FormatError("Module %s requires an instance name" % node.module)

        current = self.stackInstanceFrame(nodename, node.module)

        scope = self.frames.getCurrent()

        paramnames = self.moduleinfotable.getParamNames(node.module)
        for paramnames_i, param in enumerate(node.parameterlist):
            self.addInstanceParameterBind(param, paramnames[paramnames_i])
            value = self.optimize(self.getTree(param.argname, current))
            paramname = paramnames[paramnames_i] if param.paramname is None else param.paramname
            if paramname not in paramnames:
                raise verror.FormatError("No such parameter: %s in %s" %
                                         (paramname, nodename))
            name = scope + ScopeLabel(paramname, 'signal')
            self.setConstant(name, value)
            definition = Parameter(paramname, str(value.value))
            term = self.makeConstantTerm(name, definition, current)
            self.setConstantTerm(name, term)

        ioports = self.moduleinfotable.getIOPorts(node.module)
        for ioport_i, port in enumerate(node.portlist):
            if port.portname is not None and not (port.portname in ioports):
                raise verror.FormatError("No such port: %s in %s" %
                                          (port.argname.name, nodename))
            self.addInstancePortBind(port, ioports[ioport_i], arrayindex)

        new_current = self.frames.getCurrent()
        self.copyFrameInfo(new_current)

        self.visit(self.moduleinfotable.getDefinition(node.module))
        self.frames.setCurrent(current)

    def _visit_Instance_primitive(self, node, arrayindex=None):
        primitive_type = primitives[node.module]
        left = node.portlist[0].argname
        if arrayindex is not None:
            left = Pointer(left, IntConst(str(arrayindex)))
        right = None
        if primitive_type == None:
            right = (Pointer(node.portlist[1].argname, IntConst('0')) if arrayindex is None else
                     Pointer(node.portlist[1].argname, IntConst(str(arrayindex))))
        elif primitive_type == Unot:
            right = (Ulnot(Pointer(node.portlist[1].argname, IntConst('0'))) if arrayindex is None else
                     Ulnot(Pointer(node.portlist[1].argname, IntConst(str(arrayindex)))))
        else:
            concat_list = ([Pointer(p.argname, IntConst('0')) for p in node.portlist[1:]] if arrayindex is None else
                           [Pointer(p.argname, IntConst(str(arrayindex))) for p in node.portlist[1:]])
            right = primitive_type(Concat(concat_list))
        self.addBind(left, right, bindtype='assign')

    def visit_Initial(self, node):
        pass
        #label = self.labels.get( self.frames.getLabelKey('initial') )
        #current = self.stackNextFrame(label, 'initial',
        #                              generate=self.frames.isGenerate(),
        #                              initial=True)
        #self.generic_visit(node)
        #self.frames.setCurrent(current)

    def visit_Always(self, node):
        label = self.labels.get( self.frames.getLabelKey('always') )
        current = self.stackNextFrame(label, 'always',
                                      generate=self.frames.isGenerate(),
                                      always=True)

        (clock_name, clock_edge, clock_bit,
         reset_name, reset_edge, reset_bit,
         senslist) = self._createAlwaysinfo(node, current)

        self.frames.setAlwaysInfo(clock_name, clock_edge, clock_bit,
                                  reset_name, reset_edge, reset_bit, senslist)

        self.generic_visit(node)
        self.frames.setCurrent(current)

    def _get_signal_name(self, n):
        if isinstance(n, Identifier):
            return n.name
        if isinstance(n, Pointer):
            return self._get_signal_name(n.var)
        if isinstance(n, Partselect):
            return self._get_signal_name(n.var)
        return None

    def _createAlwaysinfo(self, node, scope):
        sens = None
        senslist = []
        clock_edge = None
        clock_name = None
        clock_bit = None
        reset_edge = None
        reset_name = None
        reset_bit = None

        for l in node.sens_list.list:
            if l.sig is None:
                continue
            if isinstance(l.sig, pyverilog.vparser.ast.Pointer):
                signame = self._get_signal_name(l.sig.var)
                bit = int(l.sig.ptr.value)
            else:
                signame = self._get_signal_name(l.sig)
                bit = 0
            if signaltype.isClock(signame):
                clock_name = self.searchTerminal(signame, scope)
                clock_edge = l.type
                clock_bit = bit
            elif signaltype.isReset(signame):
                reset_name = self.searchTerminal(signame, scope)
                reset_edge = l.type
                reset_bit = bit
            else:
                senslist.append(l)

        if clock_edge is not None and len(senslist) > 0:
            raise verror.FormatError('Illegal sensitivity list')
        if reset_edge is not None and len(senslist) > 0:
            raise verror.FormatError('Illegal sensitivity list')

        return (clock_name, clock_edge, clock_bit, reset_name, reset_edge, reset_bit, senslist)

    def visit_IfStatement(self, node):
        if self.frames.isFunctiondef() and not self.frames.isFunctioncall(): return
        if self.frames.isTaskdef() and not self.frames.isTaskcall(): return

        if (self.frames.isGenerate() and
            not self.frames.isAlways() and not self.frames.isInitial() and
            not self.frames.isFunctioncall() and not self.frames.isTaskcall() and
            not self.frames.isFunctiondef() and not self.frames.isTaskdef()):
            # generate-if statement
            current = self.frames.getCurrent()
            tree = self.getTree(node.cond, current)
            rslt = self.optimize(tree)
            if not isinstance(rslt, DFEvalValue):
                raise verror.FormatError("Can not resolve generate-if condition")
            if rslt.value > 0:
                label = self._if_true(node)
                if node.true_statement is not None:
                    self.copyBlockingAssigns(current + ScopeLabel(label, 'if'), current)
            else:
                label = self.labels.get( self.frames.getLabelKey('if') )
                self._if_false(node, label)
                if node.false_statement is not None:
                    self.copyBlockingAssigns(current + ScopeLabel(self._toELSE(label), 'if'), current)
            return

        label = self._if_true(node)
        self._if_false(node, label)

        current = self.frames.getCurrent()
        if node.true_statement is not None:
            self.copyBlockingAssigns(current + ScopeLabel(label, 'if'), current)
        if node.false_statement is not None:
            self.copyBlockingAssigns(current + ScopeLabel(self._toELSE(label), 'if'), current)

    def _toELSE(self, label):
        return label + '_ELSE'

    def _if_true(self, node):
        if node.true_statement is None: return None
        label = self.labels.get( self.frames.getLabelKey('if') )
        current = self.stackNextFrame(label, 'if',
                                      frametype='ifthen',
                                      condition=node.cond,
                                      functioncall=self.frames.isFunctioncall(),
                                      taskcall=self.frames.isTaskcall(),
                                      generate=self.frames.isGenerate(),
                                      always=self.frames.isAlways(),
                                      initial=self.frames.isInitial())

        self.copyPreviousNonblockingAssign(current + ScopeLabel(label, 'if'))

        if node.true_statement is not None: self.visit(node.true_statement)
        self.frames.setCurrent(current)
        return label

    def _if_false(self, node, label):
        if node.false_statement is None: return
        label = self._toELSE(label)
        current = self.stackNextFrame(label, 'if',
                                      frametype='ifelse',
                                      condition=node.cond,
                                      functioncall=self.frames.isFunctioncall(),
                                      taskcall=self.frames.isTaskcall(),
                                      generate=self.frames.isGenerate(),
                                      always=self.frames.isAlways(),
                                      initial=self.frames.isInitial())

        self.copyPreviousNonblockingAssign(current + ScopeLabel(label, 'if'))

        if node.false_statement is not None: self.visit(node.false_statement)
        self.frames.setCurrent(current)
        return label

    def visit_CaseStatement(self, node):
        if self.frames.isFunctiondef() and not self.frames.isFunctioncall(): return
        if self.frames.isTaskdef() and not self.frames.isTaskcall(): return
        start_frame = self.frames.getCurrent()
        caseframes = []
        self._case(node.comp, node.caselist, caseframes)
        self.frames.setCurrent(start_frame)
        for f in caseframes:
            self.copyBlockingAssigns(f, start_frame)

    def visit_CasexStatement(self, node):
        return self.visit_CaseStatement(node)
        
    def _case(self, comp, caselist, myframes):
        if len(caselist) == 0: return

        case = caselist[0]
        cond = Eq(comp, case)
        label = self.labels.get( self.frames.getLabelKey('if') )
        current = self.stackNextFrame(label, 'if',
                                      frametype='ifthen',
                                      condition=cond,
                                      functioncall=self.frames.isFunctioncall(),
                                      taskcall=self.frames.isTaskcall(),
                                      generate=self.frames.isGenerate(),
                                      always=self.frames.isAlways(),
                                      initial=self.frames.isInitial())

        self.copyPreviousNonblockingAssign(current + ScopeLabel(label, 'if'))

        myframes.append( self.frames.getCurrent() )

        if case.statement is not None:
            self.visit(case.statement)
        self.frames.setCurrent(current)

        if len(caselist) == 1: return

        label = self._toELSE(label)
        current = self.stackNextFrame(label, 'if',
                                      frametype='ifelse',
                                      condition=cond,
                                      functioncall=self.frames.isFunctioncall(),
                                      taskcall=self.frames.isTaskcall(),
                                      generate=self.frames.isGenerate(),
                                      always=self.frames.isAlways(),
                                      initial=self.frames.isInitial())

        self.copyPreviousNonblockingAssign(current + ScopeLabel(label, 'if'))

        myframes.append( current + ScopeLabel(label, 'if') )

        self._case(comp, caselist[1:], myframes)

    def visit_ForStatement(self, node):
        if self.frames.isFunctiondef() and not self.frames.isFunctioncall(): return
        if self.frames.isTaskdef() and not self.frames.isTaskcall(): return
        ## pre-statement
        current = self.frames.getCurrent()
        pre_right = self.getTree(node.pre.right, current)
        pre_right_value = self.optimize(pre_right)
        loop = pre_right_value.value
        self.frames.setForPre()
        self.visit(node.pre)
        self.frames.unsetForPre()
        label = self.labels.get( self.frames.getLabelKey('for') )
        #loop = 0
        start_frame = self.frames.getCurrent()
        while True:
            ## cond-statement
            current = self.frames.getCurrent()
            raw_tree = self.getTree(node.cond, current)
            rslt = self.optimize(raw_tree)
            if not isinstance(rslt, DFEvalValue):
                raise verror.FormatError(("Can not process the for-statement. "
                                          "for-condition should be evaluated statically."))
            if rslt.value <= 0: break

            ## main-statement
            current = self.stackNextFrame(label, 'for',
                                          frametype='for',
                                          functioncall=self.frames.isFunctioncall(),
                                          taskcall=self.frames.isTaskcall(),
                                          generate=self.frames.isGenerate(),
                                          always=self.frames.isAlways(),
                                          initial=self.frames.isInitial(),
                                          loop=loop, loop_iter=self.frames.getForIter())

            self.visit(node.statement)
            self.copyBlockingAssigns(self.frames.getCurrent(), start_frame)
            self.frames.setCurrent(current)

            ## post-statement
            current = self.frames.getCurrent()
            post_right = self.getTree(node.post.right, current)
            post_right_value = self.optimize(post_right)
            loop = post_right_value.value
            self.frames.setForPost()
            self.visit(node.post)
            self.frames.unsetForPost()
            #loop += 1

    def visit_WhileStatement(self, node):
        if self.frames.isFunctiondef() and not self.frames.isFunctioncall(): return
        if self.frames.isTaskdef() and not self.frames.isTaskcall(): return
        label = self.labels.get( self.frames.getLabelKey('while') )
        loop = 0
        start_frame = self.frames.getCurrent()
        while True:
            ## cond-statement
            current = self.frames.getCurrent()
            raw_tree = self.getTree(node.cond, current)
            rslt = self.optimize(raw_tree)
            if not isinstance(rslt, DFEvalValue):
                raise verror.FormatError(("Can not process the while-statement. "
                                          "while-condition should be evaluated statically."))
            if rslt.value <= 0: break

            ## main-statement
            current = self.stackNextFrame(label, 'while',
                                          frametype='while',
                                          functioncall=self.frames.isFunctioncall(),
                                          taskcall=self.frames.isTaskcall(),
                                          generate=self.frames.isGenerate(),
                                          always=self.frames.isAlways(),
                                          initial=self.frames.isInitial(),
                                          loop=loop)

            self.visit(node.statement)
            self.copyBlockingAssigns(self.frames.getCurrent(), start_frame)
            self.frames.setCurrent(current)
            loop += 1

    def visit_GenerateStatement(self, node):
        label = self.labels.get( self.frames.getLabelKey('generate') )
        current = self.stackNextFrame(label, 'generate',
                                      generate=True)

        self.generic_visit(node)
        self.frames.setCurrent(current)

    def visit_Block(self, node):
        label = None
        if node.scope is not None:
            label = node.scope
        else:
            label = self.labels.get( self.frames.getLabelKey('block') )
        current = self.stackNextFrame(label, 'block',
                                      frametype='block',
                                      functioncall=self.frames.isFunctioncall(),
                                      taskcall=self.frames.isTaskcall(),
                                      generate=self.frames.isGenerate(),
                                      always=self.frames.isAlways(),
                                      initial=self.frames.isInitial())

        self.generic_visit(node)
        self.frames.setCurrent(current)
        if self.frames.isAlways():
            self.copyBlockingAssigns(current + ScopeLabel(label, 'block'), current)

    def visit_Assign(self, node):
        self.addBind(node.left, node.right, bindtype='assign')

    def visit_BlockingSubstitution(self, node):
        self.addBind(node.left, node.right, self.frames.getAlwaysStatus(), 'blocking')

    def visit_NonblockingSubstitution(self, node):
        if self.frames.isForpre() or self.frames.isForpost():
            raise verror.FormatError(("Non Blocking Substitution is not allowed"
                                      "in for-statement"))
        if self.frames.isFunctioncall():
            raise verror.FormatError("Non Blocking Substitution is not allowed in function")
        self.addBind(node.left, node.right, self.frames.getAlwaysStatus(), 'nonblocking')

    def visit_SystemCall(self, node):
        print("Warning: Isolated system call is not supported: %s" % node.syscall)

    ############################################################################
    def optimize(self, node):
        return self.optimizer.optimize(node)

    def stackInstanceFrame(self, instname, modulename):
        current = self.frames.getCurrent()
        self.frames.setCurrent(current + ScopeLabel(instname, 'module'))
        self.copyFrameInfo(current + ScopeLabel(instname, 'module'))
        return current

    def stackNextFrame(self, label, scopetype,
                       frametype='none',
                       alwaysinfo=None, condition=None,
                       module=False, functioncall=False, taskcall=False,
                       generate=False, always=False, initial=False, loop=None, loop_iter=None):
        current = self.frames.getCurrent()
        scopelabel = ScopeLabel(label, scopetype, loop)
        nextscope = current + scopelabel
        if not self.frames.hasFrame(nextscope):
            current = self.frames.addFrame(scopelabel,
                                           frametype=frametype,
                                           alwaysinfo=alwaysinfo, condition=condition,
                                           module=module, functioncall=functioncall, taskcall=taskcall,
                                           generate=generate, always=always, initial=initial,
                                           loop=loop, loop_iter=loop_iter)
        self.frames.setCurrent(nextscope)
        new_current = self.frames.getCurrent()
        self.copyFrameInfo(new_current)
        return current

    def copyFrameInfo(self, current):
        for name, definitions in self.frames.getConsts(current).items():
            if len(definitions) > 1:
                raise verror.FormatError("Multiple definitions for Constant")
            for definition in definitions:
                termtype = definition.__class__.__name__
                term = Term(name, set([termtype]))
                self.dataflow.addTerm(name, term)

        for name, definitions in self.frames.getConsts(current).items():
            if len(definitions) > 1:
                raise verror.FormatError("Multiple definitions for Constant")
            for definition in definitions:
                cterm = self.makeConstantTerm(name, definition, current)
                self.setConstantTerm(name, cterm)

        all_passed = False
        while not all_passed:
            all_passed = True
            for name, definitions in self.frames.getConsts(current).items():
                if len(definitions) > 1:
                    raise verror.FormatError("Multiple definitions for Constant")
                if self.hasConstant(name):
                    continue
                for definition in definitions:
                    if isinstance(definition, Genvar):
                        continue
                    value = self.optimize(self.getTree(definition.value, current))
                    if not isinstance(value, DFEvalValue):
                        all_passed = False
                        continue
                    self.setConstant(name, value)

    def copyAllFrameInfo(self):
        for name, definitions in self.frames.getAllConsts().items():
            if len(definitions) > 1:
                raise verror.FormatError("Multiple definitions for Constant")
            for definition in definitions:
                termtype = definition.__class__.__name__
                term = Term(name, set([termtype]))
                self.dataflow.addTerm(name, term)

        for name, definitions in self.frames.getAllSignals().items():
            for definition in definitions:
                termtype = definition.__class__.__name__
                self.dataflow.addTerm(name, Term(name, set([termtype])))

        for name, definition in self.frames.getAllFunctions().items():
            self.dataflow.addFunction(name, definition.getDefinition())
            self.dataflow.addFunctionPorts(name, definition.getIOPorts())

        for name, definition in self.frames.getAllTasks().items():
            self.dataflow.addTask(name, definition.getDefinition())
            self.dataflow.addTaskPorts(name, definition.getIOPorts())

    def copyPreviousNonblockingAssign(self, scope):
        assign = self.frames.getPreviousNonblockingAssign()
        for name, bindlist in assign.items():
            for bind in bindlist:
                self.frames.addNonblockingAssign(name, bind)
                msb = bind.msb
                lsb = bind.lsb
                ptr = bind.ptr
                part_msb = None
                part_lsb = None
                condlist, flowlist = self.getCondflow(scope)
                alwaysinfo = bind.alwaysinfo
                raw_tree = bind.tree
                new_bind = self.makeBind(name, msb, lsb, ptr, part_msb, part_lsb,
                                         raw_tree, condlist, flowlist,
                                         alwaysinfo=alwaysinfo)
                self.dataflow.addBind(name, new_bind)

    def copyBlockingAssigns(self, scope_copy_from, scope_copy_to):
        assign = self.frames.getBlockingAssignsScope(scope_copy_from)
        for name, bindlist in assign.items():
            for bind in bindlist:
                self.frames.setBlockingAssign(name, bind, scope_copy_to)

    def setConstant(self, name, value):
        self.optimizer.setConstant(name, value)

    def resetConstant(self, name):
        self.optimizer.resetConstant(name)

    def getConstant(self, name):
        return self.optimizer.getConstant(name)

    def hasConstant(self, name):
        return self.optimizer.hasConstant(name)

    def setConstantTerm(self, name, term):
        self.optimizer.setTerm(name, term)

    def getTerm(self, name):
        term = self.dataflow.getTerm(name)
        return term

    def getTermWidth(self, name):
        term = self.dataflow.getTerm(name)
        return term.msb, term.lsb

    def getTermLength(self, name):
        term = self.dataflow.getTerm(name)
        return term.lenmsb, term.lenlsb

    def getTermtype(self, name):
        term = self.dataflow.getTerm(name)
        return term.termtype

    def getBindlist(self, name):
        return self.dataflow.getBindlist(name)

    def renameVar(self, name):
        renamedvar = (name[:-1] +
                      ScopeLabel('_rn' + str(self.renamecnt) +
                                 '_' + name[-1].scopename, 'signal'))
        self.renamecnt += 1
        return renamedvar

    ############################################################################
    def toScopeChain(self, blocklabel, current):
        scopelist = []
        for b in blocklabel.labellist:
            if b.loop is not None:
                loop = self.optimize(self.getTree(b.loop, current))
                if not isinstance(loop, DFEvalValue):
                    raise verror.FormatError('Loop iterator should be constant')
                scopelist.append( ScopeLabel('for', 'for', loop.value) )
            scopelist.append( ScopeLabel(b.name, 'any') )
        return ScopeChain( scopelist )

    def searchConstantDefinition(self, key, name):
        const = self.frames.searchConstantDefinition(key, name)
        if const is None: raise verror.DefinitionError('constant value not found: %s' % name)
        return const

    def getModuleScopeChain(self, target):
        remove_cnt = 0
        length = len(target)
        for scope in reversed(target):
            if scope.scopetype == 'module':
                return target[:length-remove_cnt]
            remove_cnt += 1
        raise verror.DefinitionError('module not found')

    def searchScopeConstantDefinition(self, blocklabel, name, current):
        currentmodule = self.getModuleScopeChain(current)
        localchain = currentmodule + self.toScopeChain(blocklabel, current)
        matchedchain = self.frames.searchMatchedScopeChain(currentmodule, localchain)
        constdef = self.frames.getConstantDefinition(matchedchain, name)
        return constdef

    def searchScopeTerminal(self, blocklabel, name, current):
        currentmodule = self.getModuleScopeChain(current)
        localchain = currentmodule + self.toScopeChain(blocklabel, current)
        matchedchain = self.frames.searchMatchedScopeChain(currentmodule, localchain)
        varname = matchedchain + ScopeLabel(name, 'signal')
        if self.dataflow.hasTerm(varname): return varname
        raise verror.DefinitionError('No such signal: %s' % varname)

    def searchScopeFunction(self, blocklabel, name, current):
        currentmodule = self.getModuleScopeChain(current)
        localchain = currentmodule + self.toScopeChain(blocklabel, current)
        matchedchain = self.frames.searchMatchedScopeChain(currentmodule, localchain)
        varname = matchedchain + ScopeLabel(name, 'function')
        if self.dataflow.hasFunction(varname): return self.dataflow.getFunction(varname)
        raise verror.DefinitionError('No such function: %s' % varname)

    def searchScopeTask(self, blocklabel, name, current):
        currentmodule = self.getModuleScopeChain(current)
        localchain = currentmodule + self.toScopeChain(blocklabel, current)
        matchedchain = self.frames.searchMatchedScopeChain(currentmodule, localchain)
        varname = matchedchain + ScopeLabel(name, 'task')
        if self.dataflow.hasTask(varname): return self.dataflow.getTask(varname)
        raise verror.DefinitionError('No such task: %s' % varname)

    def searchScopeFunctionPorts(self, blocklabel, name, current):
        currentmodule = self.getModuleScopeChain(current)
        localchain = currentmodule + self.toScopeChain(blocklabel, current)
        matchedchain = self.frames.searchMatchedScopeChain(currentmodule, localchain)
        varname = matchedchain + ScopeLabel(name, 'function')
        if self.dataflow.hasFunction(varname):
            return self.dataflow.getFunctionPorts(varname)
        raise verror.DefinitionError('No such function: %s' % varname)

    def searchScopeTaskPorts(self, blocklabel, name, current):
        currentmodule = self.frames.getModuleScopeChain(current)
        localchain = currentmodule + self.toScopeChain(blocklabel, current)
        matchedchain = self.frames.searchMatchedScopeChain(currentmodule, localchain)
        varname = matchedchain + ScopeLabel(name, 'task')
        if self.dataflow.hasTask(varname):
            return self.dataflow.getTaskPorts(varname)
        raise verror.DefinitionError('No such task: %s' % varname)

    def searchTerminal(self, name, scope):
        if len(scope) == 0: return None
        varname = scope + ScopeLabel(name, 'signal')
        if self.dataflow.hasTerm(varname): return varname
        if self.frames.dict[scope].isModule(): return None
        #if self.frames.dict[scope].isFunctioncall(): return None
        return self.searchTerminal(name, scope[:-1])

    def searchFunction(self, name, scope):
        if len(scope) == 0: return None
        varname = scope + ScopeLabel(name, 'function')
        if self.dataflow.hasFunction(varname): return self.dataflow.getFunction(varname)
        if self.frames.dict[scope].isModule(): return None
        return self.searchFunction(name, scope[:-1])

    def searchTask(self, name, scope):
        if len(scope) == 0: return None
        varname = scope + ScopeLabel(name, 'task')
        if self.dataflow.hasTask(varname): return self.dataflow.getTask(varname)
        if self.frames.dict[scope].isModule(): return None
        return self.searchTask(name, scope[:-1])

    def searchFunctionPorts(self, name, scope):
        if len(scope) == 0: return ()
        varname = scope + ScopeLabel(name, 'function')
        if self.dataflow.hasFunction(varname):
            return self.dataflow.getFunctionPorts(varname)
        if self.frames.dict[scope].isModule(): return ()
        return self.searchFunctionPorts(name, scope[:-1])

    def searchTaskPorts(self, name, scope):
        if len(scope) == 0: return ()
        varname = scope + ScopeLabel(name, 'task')
        if self.dataflow.hasTask(varname):
            return self.dataflow.getTaskPorts(varname)
        if self.frames.dict[scope].isModule(): return ()
        return self.searchTaskPorts(name, scope[:-1])

    ############################################################################
    def makeConstantTerm(self, name, node, scope):
        termtype = node.__class__.__name__
        termtypes = set([termtype])
        msb = DFIntConst('31') if node.width is None else self.makeDFTree(node.width.msb, scope)
        lsb = DFIntConst('0') if node.width is None else self.makeDFTree(node.width.lsb, scope)
        return Term(name, termtypes, msb, lsb)

    def addTerm(self, node, rscope=None):
        if self.frames.isFunctiondef() and not self.frames.isFunctioncall(): return
        if self.frames.isTaskdef() and not self.frames.isTaskcall(): return
        scope = self.frames.getCurrent() if rscope is None else rscope
        name = scope + ScopeLabel(node.name, 'signal')
        termtype = node.__class__.__name__
        if self.frames.isFunctioncall(): termtype = 'Function'
        if self.frames.isTaskcall(): termtype = 'Task'
        termtypes = set([termtype])

        if isinstance(node, (Parameter, Localparam)):
            msb = DFIntConst('31') if node.width is None else self.makeDFTree(node.width.msb, scope)
        else:
            msb = DFIntConst('0') if node.width is None else self.makeDFTree(node.width.msb, scope)
        lsb = DFIntConst('0') if node.width is None else self.makeDFTree(node.width.lsb, scope)
        
        lenmsb = None
        lenlsb = None
        if isinstance(node, RegArray) or isinstance(node, WireArray):
            lenmsb = self.makeDFTree(node.length.msb, scope)
            lenlsb = self.makeDFTree(node.length.lsb, scope)

        term = Term(name, termtypes, msb, lsb, lenmsb, lenlsb)
        self.dataflow.addTerm(name, term)
        self.setConstantTerm(name, term)

    def addBind(self, left, right, alwaysinfo=None, bindtype=None):
        if self.frames.isFunctiondef() and not self.frames.isFunctioncall(): return
        if self.frames.isTaskdef() and not self.frames.isTaskcall(): return
        lscope = self.frames.getCurrent()
        rscope = lscope
        dst = self.getDestinations(left, lscope)

        if bindtype == 'blocking':
            self.addDataflow_blocking(dst, right, lscope, rscope, alwaysinfo)
        else:
            self.addDataflow(dst, right, lscope, rscope, alwaysinfo, bindtype)

    def addInstanceParameterBind(self, param, name=None):
        lscope = self.frames.getCurrent()
        rscope = lscope[:-1]
        paramname = name if param.paramname is None else param.paramname
        dst = self.getDestinations(paramname, lscope)

        self.addDataflow(dst, param.argname, lscope, rscope, None, 'parameter')

    def addInstancePortBind(self, port, instportname=None, arrayindex=None):
        lscope = self.frames.getCurrent()
        rscope = lscope[:-1]
        portname = instportname if port.portname is None else port.portname
        ldst = self.getDestinations(portname, lscope)
        if ldst[0][0] is None:
            raise verror.DefinitionError('No such port: %s' % portname)

        termtype = self.getTermtype(ldst[0][0])
        for t in termtype:
            if t == 'Input':
                if port.argname is None: continue
                portarg = (port.argname if arrayindex is None else
                           Pointer(port.argname, IntConst(str(arrayindex))))
                self.addDataflow(ldst, portarg, lscope, rscope)
            elif t == 'Output':
                if port.argname is None: continue
                portarg = (port.argname if arrayindex is None else
                           Pointer(port.argname, IntConst(str(arrayindex))))
                rdst = self.getDestinations(portarg, rscope)
                self.addDataflow(rdst, portname, rscope, lscope)
            elif t == 'Inout':
                if port.argname is None: continue
                portarg = (port.argname if arrayindex is None else
                           Pointer(port.argname, IntConst(str(arrayindex))))
                self.addDataflow(ldst, portarg, lscope, rscope)
                rdst = self.getDestinations(portarg, rscope)
                self.addDataflow(rdst, portname, rscope, lscope)

    ############################################################################
    def addDataflow(self, dst, right, lscope, rscope, alwaysinfo=None, bindtype=None):
        condlist, flowlist = self.getCondflow(lscope)
        raw_tree = self.getTree(right, rscope)
        self.setDataflow(dst, raw_tree, condlist, flowlist, alwaysinfo, bindtype)

    def addDataflow_blocking(self, dst, right, lscope, rscope, alwaysinfo):
        condlist, flowlist = self.getCondflow(lscope)
        raw_tree = self.getTree(right, rscope)

        self.setDataflow_rename(dst, raw_tree, condlist, flowlist, lscope, alwaysinfo)

        if len(dst) == 1: # set genvar value to the constant table
            name = dst[0][0]
            if signaltype.isGenvar(self.getTermtype(name)):
                value = self.optimize(raw_tree)
                self.setConstant(name, value)
            else: # for "for-statement"
                value = self.optimize(raw_tree)
                if isinstance(value, DFEvalValue):
                    self.setConstant(name, value)
                else:
                    self.resetConstant(name)

    ############################################################################
    def getCondflow(self, scope):
        condlist = self.getCondlist(scope)
        condlist = self.resolveCondlist(condlist, scope)
        flowlist = self.getFlowlist(scope)
        return (condlist, flowlist)

    def getCondlist(self, scope):
        ret = []
        s = scope
        while s is not None:
            frame = self.frames.dict[s]
            cond = frame.getCondition()
            if cond is not None:
                ret.append(self.makeDFTree(cond, self.reduceIfScope(s)))
            if frame.isModule(): break
            if frame.isFunctioncall(): break
            s = frame.previous
        ret.reverse()
        return tuple(ret)

    def getFlowlist(self, scope):
        ret = []
        s = scope
        while s is not None:
            frame = self.frames.dict[s]
            cond = frame.getCondition()
            if cond is not None: ret.append(not frame.isIfelse())
            if frame.isModule(): break
            if frame.isFunctioncall(): break
            s = frame.previous
        ret.reverse()
        return tuple(ret)

    ############################################################################
    def getTree(self, node, scope):
        expr = node.var if isinstance(node, Rvalue) else node
        tree = self.makeDFTree(expr, scope)
        tree = self.resolveBlockingAssign(tree, scope)
        if not self.noreorder: tree = reorder.reorder(tree)
        return tree

    def makeDFTree(self, node, scope):
        if isinstance(node, str):
            name = self.searchTerminal(node, scope)
            return DFTerminal(name)

        if isinstance(node, Identifier):
            if node.scope is not None:
                name = self.searchScopeTerminal(node.scope, node.name, scope)
                if name is None:
                    raise verror.DefinitionError('No such signal: %s' % node.name)
                return DFTerminal(name)
            name = self.searchTerminal(node.name, scope)
            if name is None:
                raise verror.DefinitionError('No such signal: %s' % node.name)
            return DFTerminal(name)

        if isinstance(node, IntConst):
            return DFIntConst(node.value)

        if isinstance(node, FloatConst):
            return DFFloatConst(node.value)

        if isinstance(node, StringConst):
            return DFStringConst(node.value)

        if isinstance(node, Cond):
            true_df = self.makeDFTree(node.true_value, scope)
            false_df = self.makeDFTree(node.false_value, scope)
            cond_df = self.makeDFTree(node.cond, scope)
            if isinstance(cond_df, DFBranch):
                return reorder.insertCond(cond_df, true_df, false_df)
            return DFBranch(cond_df, true_df, false_df)

        if isinstance(node, UnaryOperator):
            right_df = self.makeDFTree(node.right, scope)
            if isinstance(right_df, DFBranch):
                return reorder.insertUnaryOp(right_df, node.__class__.__name__)
            return DFOperator((right_df,), node.__class__.__name__)

        if isinstance(node, Operator):
            left_df = self.makeDFTree(node.left, scope)
            right_df = self.makeDFTree(node.right, scope)
            if isinstance(left_df, DFBranch) or isinstance(right_df, DFBranch):
                return reorder.insertOp(left_df, right_df, node.__class__.__name__)
            return DFOperator((left_df, right_df,), node.__class__.__name__)

        if isinstance(node, Partselect):
            var_df = self.makeDFTree(node.var, scope)
            msb_df = self.makeDFTree(node.msb, scope)
            lsb_df = self.makeDFTree(node.lsb, scope)

            if isinstance(var_df, DFBranch):
                return reorder.insertPartselect(var_df, msb_df, lsb_df)
            return DFPartselect(var_df, msb_df, lsb_df)

        if isinstance(node, Pointer):
            var_df = self.makeDFTree(node.var, scope)
            ptr_df = self.makeDFTree(node.ptr, scope)

            if (isinstance(var_df, DFTerminal) and
                (signaltype.isRegArray(self.getTermtype(var_df.name)) or
                 signaltype.isWireArray(self.getTermtype(var_df.name)))):
                return DFPointer(var_df, ptr_df)
            return DFPartselect(var_df, ptr_df, copy.deepcopy(ptr_df))

        if isinstance(node, Concat):
            nextnodes = []
            for n in node.list:
                nextnodes.append(self.makeDFTree(n, scope))
            for n in nextnodes:
                if isinstance(n, DFBranch):
                    return reorder.insertConcat(tuple(nextnodes))
            return DFConcat(tuple(nextnodes))

        if isinstance(node, Repeat):
            nextnodes = []
            times = self.optimize(self.getTree(node.times, scope)).value
            value = self.makeDFTree(node.value, scope)
            for i in range(int(times)):
                nextnodes.append(copy.deepcopy(value))
            return DFConcat(tuple(nextnodes))

        if isinstance(node, FunctionCall):
            func = self.searchFunction(node.name.name, scope)
            if func is None:
                raise verror.DefinitionError('No such function: %s' % node.name.name)
            label = self.labels.get( self.frames.getLabelKey('functioncall') )

            save_current = self.frames.getCurrent()
            self.frames.setCurrent(scope)

            current = self.frames.addFrame(
                ScopeLabel(label, 'functioncall'),
                functioncall=True, generate=self.frames.isGenerate(),
                always=self.frames.isAlways())

            varname = self.frames.getCurrent() + ScopeLabel(func.name,'signal')

            self.addTerm( Wire(func.name, func.retwidth) )

            funcports = self.searchFunctionPorts(node.name.name, scope)
            funcargs = node.args

            if len(funcports) != len(funcargs):
                raise verror.FormatError("%s takes exactly %d arguments. (%d given)" %
                                         (func.name.name, len(funcports), len(funcargs)))
            for port in funcports:
                self.addTerm( Wire(port.name, port.width) )

            lscope = self.frames.getCurrent()
            rscope = scope
            func_i = 0
            for port in funcports:
                arg = funcargs[func_i]
                dst = self.getDestinations(port.name, lscope)
                self.addDataflow(dst, arg, lscope, rscope)
                func_i += 1

            self.visit(func)

            self.frames.setCurrent(current)
            self.frames.setCurrent(save_current)

            return DFTerminal(varname)

        if isinstance(node, TaskCall):
            task = self.searchTask(node.name.name, scope)
            label = self.labels.get( self.frames.getLabelKey('taskcall') )

            current = self.frames.addFrame(
                ScopeLabel(label, 'taskcall'),
                taskcall=True, generate=self.frames.isGenerate(),
                always=self.frames.isAlways())

            varname = self.frames.getCurrent() + ScopeLabel(task.name,'signal')

            taskports = self.searchTaskPorts(node.name.name, scope)
            taskargs = node.args

            if len(taskports) != len(taskargs):
                raise verror.FormatError("%s takes exactly %d arguments. (%d given)" %
                                         (task.name.name, len(taskports), len(taskargs)))
            for port in taskports:
                self.addTerm( Wire(port.name, port.width) )

            lscope = self.frames.getCurrent()
            rscope = scope
            task_i = 0
            for port in taskports:
                arg = taskargs[task_i]
                dst = self.getDestinations(port.name, lscope)
                self.addDataflow(dst, arg, lscope, rscope)
                task_i += 1

            self.visit(taskargs)
            self.frames.setCurrent(current)
            return DFTerminal(varname)

        if isinstance(node, SystemCall):
            if node.syscall == 'unsigned':
                return self.makeDFTree(node.args[0], scope)
            if node.syscall == 'signed':
                return self.makeDFTree(node.args[0], scope)
            return DFIntConst('0')

        raise verror.FormatError("unsupported AST node type: %s %s" %
                                 (str(type(node)), str(node)))

    ############################################################################
    def reduceIfScope(self, scope):
        if len(scope) == 0: return scope
        if scope[-1].scopetype == 'if': return scope[:-1]
        return self.reduceIfScope(scope[:-1])

    def resolveCondlist(self, condlist, scope):
        resolved_condlist = []
        reducedscope = self.reduceIfScope(scope)
        for i, c in enumerate(reversed(condlist)):
            resolved_condlist.append(self.resolveBlockingAssign(c, reducedscope))
            reducedscope = self.reduceIfScope(reducedscope)
        return tuple(reversed(resolved_condlist))

    def removeOverwrappedCondition(self, tree, current_bindlist, scope):
         msb, lsb = self.getTermWidth(tree.name)
         merged_tree = self.getFitTree(current_bindlist, msb, lsb)
         condlist, flowlist = self.getCondflow(scope)
         (merged_tree,
         rest_condlist,
         rest_flowlist,
         match_flowlist) = self.diffBranchTree(merged_tree, condlist, flowlist)
         return replace.replaceUndefined(merged_tree, tree.name)

    def resolveBlockingAssign(self, tree, scope):
        if tree is None: return None

        if isinstance(tree, DFConstant):
            return tree

        if isinstance(tree, DFEvalValue):
            return tree

        if isinstance(tree, DFTerminal):
            if signaltype.isGenvar( self.getTermtype(tree.name) ):
                return self.getConstant(tree.name)

            current_bindlist = self.frames.getBlockingAssign(tree.name, scope)
            if len(current_bindlist) == 0: return tree

            return self.removeOverwrappedCondition(tree, current_bindlist, scope)

        if isinstance(tree, DFBranch):
            truenode = self.resolveBlockingAssign(tree.truenode, scope)
            falsenode = self.resolveBlockingAssign(tree.falsenode, scope)
            condnode = self.resolveBlockingAssign(tree.condnode, scope)
            if isinstance(condnode, DFBranch):
                return reorder.insertBranch(condnode, truenode, falsenode)
            return DFBranch(condnode, truenode, falsenode)

        if isinstance(tree, DFOperator):
            resolvednodes = []
            for n in tree.nextnodes:
                resolvednodes.append(self.resolveBlockingAssign(n, scope))
            for r in resolvednodes:
                if isinstance(r, DFBranch):
                    return reorder.insertOpList(resolvednodes, tree.operator)
            return DFOperator(tuple(resolvednodes), tree.operator)

        if isinstance(tree, DFConcat):
            resolvednodes = []
            for n in tree.nextnodes:
                resolvednodes.append(self.resolveBlockingAssign(n, scope))
            for r in resolvednodes:
                if isinstance(r, DFBranch):
                    return reorder.insertConcat(resolvednodes)
            return DFConcat(tuple(resolvednodes))

        if isinstance(tree, DFPartselect):
            resolved_msb = self.resolveBlockingAssign(tree.msb, scope)
            resolved_lsb = self.resolveBlockingAssign(tree.lsb, scope)
            resolved_var = self.resolveBlockingAssign(tree.var, scope)
            if isinstance(resolved_var, DFBranch):
                return reorder.insertPartselect(resolved_var,
                                                     resolved_msb, resolved_lsb)
            return DFPartselect(resolved_var, resolved_msb, resolved_lsb)

        if isinstance(tree, DFPointer):
            resolved_ptr = self.resolveBlockingAssign(tree.ptr, scope)
            if (isinstance(tree.var, DFTerminal) and
                signaltype.isRegArray(self.getTermtype(tree.var.name))):
                current_bindlist = self.frames.getBlockingAssign(tree.var.name, scope)
                if len(current_bindlist) == 0: return DFPointer(tree.var, resolved_ptr)
                new_tree = DFPointer(tree.var, resolved_ptr)
                for bind in current_bindlist:
                    new_tree = DFBranch(DFOperator((bind.ptr, resolved_ptr), 'Eq'),
                                        bind.tree, new_tree)
                print(("Warning: "
                       "Overwrting Blocking Assignment with Reg Array is not supported"))
                return new_tree
            resolved_var = self.resolveBlockingAssign(tree.var, scope)
            if isinstance(resolved_var, DFBranch):
                return reorder.insertPointer(resolved_var, resolved_ptr)
            return DFPointer(resolved_var, resolved_ptr)

        raise verror.FormatError("unsupported DFNode type: %s %s" %
                                 (str(type(tree)), str(tree)))

    def getFitTree(self, bindlist, msb, lsb):
        optimized_msb = self.optimize(msb)
        optimized_lsb = self.optimize(lsb)
        for bind in bindlist:
            if bind.msb is None and bind.lsb is None:
                return bind.tree
            if (self.optimize(bind.msb) == optimized_msb and
                self.optimize(bind.lsb) == optimized_lsb):
                return bind.tree
        return self.getMergedTree(bindlist)

    def getMergedTree(self, bindlist):
        concatlist = []
        last_msb = -1
        last_ptr = -1
        def bindkey(x):
            lsb = 0 if x.lsb is None else x.lsb.value
            ptr = 0 if not isinstance(x.ptr, DFEvalValue) else x.ptr.value
            term = self.getTerm(x.dest)
            length = (abs(self.optimize(term.msb).value -
                          self.optimize(term.lsb).value) + 1)
            return ptr * length + lsb
        for bind in sorted(bindlist, key=bindkey):
            lsb = 0 if bind.lsb is None else bind.lsb.value
            if last_ptr != (-1 if not isinstance(bind.ptr, DFEvalValue)
                             else bind.ptr.value):
                continue
            if last_msb + 1 < lsb:
                concatlist.append(DFUndefined(last_msb-lsb-1))
            concatlist.append(bind.tree)
            last_msb = -1 if bind.msb is None else bind.msb.value
            last_ptr = -1 if not isinstance(bind.ptr, DFEvalValue) else bind.ptr.value
        return DFConcat(tuple(reversed(concatlist)))

    ############################################################################
    def getDestinations(self, left, scope):
        ret = []
        dst = self.getDsts(left, scope)
        part_offset = DFIntConst('0')
        for name, msb, lsb, ptr in reversed(dst):
            if len(dst) == 1:
                ret.append((name, msb, lsb, ptr, None, None))
                continue

            if msb is None and lsb is None:
                msb, lsb = self.getTermWidth(name)
            diff = reorder.reorder( DFOperator((msb, lsb), 'Minus') )
            part_lsb = part_offset
            part_msb = reorder.reorder(DFOperator((part_offset, diff), 'Plus'))
            part_offset = reorder.reorder(
                DFOperator((part_msb, DFIntConst('1')), 'Plus'))

            ret.append((name, msb, lsb, ptr, part_msb, part_lsb))

        return tuple(ret)

    def getDsts(self, left, scope):
        if isinstance(left, Lvalue):
            return self.getDsts(left.var, scope)
        if isinstance(left, LConcat):
            dst = []
            for n in left.list:
                dst.extend(list(self.getDsts(n, scope)))
            return tuple(dst)
        ret = (self.getDst(left, scope),)
        return ret

    def getDst(self, left, scope):
        if isinstance(left, str): # Parameter
            name = self.searchTerminal(left, scope)
            return (name, None, None, None)

        if isinstance(left, Identifier):
            name = self.searchTerminal(left.name, scope)
            if name is None:
                m = re.search('none', self.default_nettype)
                if m: raise verror.FormatError("No such signal: %s" % left.name)
                m = re.search('wire', self.default_nettype)
                if m: self.addTerm(Wire(left.name), rscope=scope)
                m = re.search('reg', self.default_nettype)
                if m: self.addTerm(Reg(left.name), rscope=scope)
                name = self.searchTerminal(left.name, scope)
            if left.scope is not None:
                name = left.scope + ScopeLabel(left.name, 'signal')
            return (name, None, None, None)

        if isinstance(left, Partselect):
            if isinstance(left.var, Pointer):
                name, msb, lsb, ptr = self.getDst(left.var, scope)
                if msb is None and lsb is None:
                    msb = self.optimize(self.makeDFTree(left.msb, scope))
                    lsb = self.optimize(self.makeDFTree(left.lsb, scope))
                else:
                    raise verror.FormatError("%s is not array" % str(name))
                return (name, msb, lsb, ptr)
            name = self.searchTerminal(left.var.name, scope)
            if left.var.scope is not None:
                name = left.var.scope + ScopeLabel(left.var.name, 'signal')
            msb = self.optimize(self.makeDFTree(left.msb, scope))
            lsb = self.optimize(self.makeDFTree(left.lsb, scope))
            return (name, msb, lsb, None)

        if isinstance(left, Pointer):
            if isinstance(left.var, Pointer):
                name, msb, lsb, ptr = self.getDst(left.var, scope)
                if msb is None and lsb is None:
                    msb = self.optimize(self.makeDFTree(left.ptr, scope))
                    lsb = copy.deepcopy(msb)
                else:
                    raise verror.FormatError("%s is not array" % str(name))
                return (name, msb, lsb, ptr)
            name = self.searchTerminal(left.var.name, scope)
            if left.var.scope is not None:
                name = left.var.scope + ScopeLabel(left.var.name, 'signal')
            ptr = self.optimize(self.makeDFTree(left.ptr, scope))
            if (signaltype.isRegArray(self.getTermtype(name)) or
                signaltype.isWireArray(self.getTermtype(name))):
                return (name, None, None, ptr)
            return (name, ptr, copy.deepcopy(ptr), None)

        raise verror.FormatError("unsupported AST node type: %s %s" %
                                 (str(type(left)), str(left)))

    ############################################################################
    def setDataflow(self, dst, raw_tree, condlist, flowlist,
                    alwaysinfo=None, bindtype=None):

        for name, msb, lsb, ptr, part_msb, part_lsb in dst:
            bind = self.makeBind(name, msb, lsb, ptr, part_msb, part_lsb,
                                 raw_tree, condlist, flowlist,
                                 num_dst=len(dst),
                                 alwaysinfo=alwaysinfo,
                                 bindtype=bindtype)

            self.dataflow.addBind(name, bind)

            if alwaysinfo is not None:
                self.setNonblockingAssign(name, dst, raw_tree,
                                          msb, lsb, ptr,
                                          part_msb, part_lsb,
                                          alwaysinfo)

    def setDataflow_rename(self, dst, raw_tree, condlist, flowlist,
                           scope, alwaysinfo=None):
        renamed_dst = self.getRenamedDst(dst)
        self.setRenamedTree(renamed_dst, raw_tree, alwaysinfo)
        self.setRenamedFlow(dst, renamed_dst, condlist, flowlist, scope, alwaysinfo)

    def setNonblockingAssign(self, name, dst, raw_tree, msb, lsb, ptr,
                             part_msb, part_lsb, alwaysinfo):
        tree = raw_tree
        if len(dst) > 1:
            tree = reorder.reorder(DFPartselect(raw_tree, part_msb, part_lsb))
        bind = Bind(tree, name, msb, lsb, ptr, alwaysinfo)
        self.frames.addNonblockingAssign(name, bind)

    ############################################################################
    def getRenamedDst(self, dst):
        renamed_dst = ()
        for d in dst:
            dname = d[0]
            samename_exists = True
            while samename_exists:
                renamed_dname = self.renameVar(dname)
                samename_exists = self.dataflow.hasTerm(renamed_dname)
            term = self.dataflow.getTerm(dname)
            newterm = copy.deepcopy(term)
            newterm.name = renamed_dname
            newterm.termtype = set(['Rename'])
            self.dataflow.addTerm(renamed_dname, newterm)
            newd = (renamed_dname,) + d[1:]
            renamed_dst += (newd,)
        return renamed_dst

    def setRenamedTree(self, renamed_dst, raw_tree, alwaysinfo):
        for name, msb, lsb, ptr, part_msb, part_lsb in renamed_dst:
            tree = raw_tree
            if len(renamed_dst) > 1: tree = reorder.reorder(
                DFPartselect(tree, part_msb, part_lsb))
            bind = Bind(tree, name, msb, lsb, ptr)
            self.dataflow.addBind(name, bind)

            value = self.optimize(tree)
            if isinstance(value, DFEvalValue): self.setConstant(name, value)
            msb, lsb = self.getTermWidth(name)
            self.setConstantTerm(name, Term(name, set(['Rename']), msb, lsb))

    def setRenamedFlow(self, dst, renamed_dst, condlist, flowlist,
                       scope, alwaysinfo=None):
        term_i = 0
        for name, msb, lsb, ptr, part_msb, part_lsb in dst:
            renamed_term = DFTerminal(renamed_dst[term_i][0])
            renamed_bind = self.makeBind(name, msb, lsb, ptr, part_msb, part_lsb,
                                         renamed_term, condlist, flowlist,
                                         num_dst=len(dst), alwaysinfo=alwaysinfo)
            self.dataflow.addBind(name, renamed_bind)
            self.frames.setBlockingAssign(name, renamed_bind, scope)
            term_i+=1

    ############################################################################
    def makeBind(self, name, msb, lsb, ptr, part_msb, part_lsb,
                 raw_tree, condlist, flowlist,
                 num_dst=1, alwaysinfo=None, bindtype=None):

        current_bindlist = self.getBindlist(name)
        current_tree = None
        current_msb = None
        current_lsb = None
        current_ptr = None

        if len(current_bindlist) > 0:
            for current_bind in current_bindlist:
                if ( current_bind.msb == msb and
                     current_bind.lsb == lsb and
                     current_bind.ptr == ptr ):
                    current_tree = current_bind.tree
                    current_msb = current_bind.msb
                    current_lsb = current_bind.lsb
                    current_ptr = current_bind.ptr
                    break

        rest_tree = current_tree
        rest_condlist = condlist
        rest_flowlist = flowlist

        match_flowlist = ()
        if ( current_msb == msb and
             current_lsb == lsb and
             current_ptr == ptr ):
            (rest_tree,
            rest_condlist,
            rest_flowlist,
            match_flowlist) = self.diffBranchTree(current_tree, condlist, flowlist)

        add_tree = self.makeBranchTree(rest_condlist, rest_flowlist, raw_tree)
        if rest_flowlist and rest_tree is not None:
            _rest_flowlist = rest_flowlist[:-1] + (not rest_flowlist[-1], )
            add_tree = self.appendBranchTree(add_tree, _rest_flowlist, rest_tree)

        tree = reorder.reorder(
            self.appendBranchTree(current_tree, match_flowlist, add_tree))

        if num_dst > 1: tree = reorder.reorder(
            DFPartselect(tree, part_msb, part_lsb))

        return Bind(tree, name, msb, lsb, ptr, alwaysinfo, bindtype)

    ############################################################################
    def diffBranchTree(self, tree, condlist, flowlist, matchflowlist=()):
        if len(condlist) == 0:
            return (tree, condlist, flowlist, matchflowlist)
        if not isinstance(tree, DFBranch):
            return (tree, condlist, flowlist, matchflowlist)
        if condlist[0] != tree.condnode:
            return (tree, condlist, flowlist, matchflowlist)
        if flowlist[0]:
            return self.diffBranchTree(
                tree.truenode, condlist[1:], flowlist[1:],
                matchflowlist+(flowlist[0],))
        else:
            return self.diffBranchTree(
                tree.falsenode, condlist[1:], flowlist[1:],
                matchflowlist+(flowlist[0],))

    def makeBranchTree(self, condlist, flowlist, node):
        if len(condlist) == 0 or len(flowlist) == 0: return node
        if len(condlist) == 1:
            if flowlist[0]: return DFBranch(condlist[0], node, None)
            else: return DFBranch(condlist[0], None, node)
        else:
            if flowlist[0]: return DFBranch(
                condlist[0],
                self.makeBranchTree(condlist[1:], flowlist[1:], node),
                None)
            else: return DFBranch(
                condlist[0],
                None,
                self.makeBranchTree(condlist[1:], flowlist[1:], node))

    def appendBranchTree(self, base, pos, tree):
        if len(pos) == 0: return tree
        if len(pos) == 1:
            if pos[0]: return DFBranch(base.condnode, tree, base.falsenode)
            else: return DFBranch(base.condnode, base.truenode, tree)
        else:
            if pos[0]: return DFBranch(
                base.condnode,
                self.appendBranchTree(base.truenode, pos[1:], tree),
                base.falsenode)
            else: return DFBranch(
                base.condnode,
                base.truenode,
                self.appendBranchTree(base.falsenode, pos[1:], tree))



import os
import sys
import subprocess

text = open('list_ast.txt', 'r').read()
lines = text.split('\n')
for line in lines:
    if line != '':
        subprocess.call('touch template/' + line.lower() + '.txt', shell=True)



from setuptools import setup, find_packages

import pyverilog.utils.version
import re
import os

m = re.search(r'(\d+\.\d+\.\d+)', pyverilog.utils.version.VERSION)
version = m.group(1) if m is not None else '0.0.0'

def read(filename):
    return open(os.path.join(os.path.dirname(__file__), filename)).read()

setup(name='pyverilog',
      version=version,
      description='Python-based Hardware Design Processing Toolkit for Verilog HDL: Parser, Dataflow Analyzer, Controlflow Analyzer and Code Generator',
      long_description=read('README.rst'),
      keywords = 'Verilog HDL, Lexer, Parser, Dataflow Analyzer, Control-flow Analyzer, Code Generator, Visualizer',
      author='Shinya Takamaeda-Yamazaki',
      author_email='shinya.takamaeda_at_gmail_com',
      license="Apache License 2.0",
      url='http://shtaxxx.github.io/Pyverilog/',
      packages=find_packages(),
      package_data={ 'pyverilog.ast_code_generator' : ['template/*'],  },
)



# -----------------------------------------------------------------------------
# ply: lex.py
#
# Copyright (C) 2001-2011,
# David M. Beazley (Dabeaz LLC)
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 
# * Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.  
# * Redistributions in binary form must reproduce the above copyright notice, 
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.  
# * Neither the name of the David Beazley or Dabeaz LLC may be used to
#   endorse or promote products derived from this software without
#  specific prior written permission. 
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
# -----------------------------------------------------------------------------

__version__    = "3.4"
__tabversion__ = "3.2"       # Version of table file used

import re, sys, types, copy, os

# This tuple contains known string types
try:
    # Python 2.6
    StringTypes = (types.StringType, types.UnicodeType)
except AttributeError:
    # Python 3.0
    StringTypes = (str, bytes)

# Extract the code attribute of a function. Different implementations
# are for Python 2/3 compatibility.

if sys.version_info[0] < 3:
    def func_code(f):
        return f.func_code
else:
    def func_code(f):
        return f.__code__

# This regular expression is used to match valid token names
_is_identifier = re.compile(r'^[a-zA-Z0-9_]+$')

# Exception thrown when invalid token encountered and no default error
# handler is defined.

class LexError(Exception):
    def __init__(self,message,s):
         self.args = (message,)
         self.text = s

# Token class.  This class is used to represent the tokens produced.
class LexToken(object):
    def __str__(self):
        return "LexToken(%s,%r,%d,%d)" % (self.type,self.value,self.lineno,self.lexpos)
    def __repr__(self):
        return str(self)

# This object is a stand-in for a logging object created by the 
# logging module.  

class PlyLogger(object):
    def __init__(self,f):
        self.f = f
    def critical(self,msg,*args,**kwargs):
        self.f.write((msg % args) + "\n")

    def warning(self,msg,*args,**kwargs):
        self.f.write("WARNING: "+ (msg % args) + "\n")

    def error(self,msg,*args,**kwargs):
        self.f.write("ERROR: " + (msg % args) + "\n")

    info = critical
    debug = critical

# Null logger is used when no output is generated. Does nothing.
class NullLogger(object):
    def __getattribute__(self,name):
        return self
    def __call__(self,*args,**kwargs):
        return self

# -----------------------------------------------------------------------------
#                        === Lexing Engine ===
#
# The following Lexer class implements the lexer runtime.   There are only
# a few public methods and attributes:
#
#    input()          -  Store a new string in the lexer
#    token()          -  Get the next token
#    clone()          -  Clone the lexer
#
#    lineno           -  Current line number
#    lexpos           -  Current position in the input string
# -----------------------------------------------------------------------------

class Lexer:
    def __init__(self):
        self.lexre = None             # Master regular expression. This is a list of
                                      # tuples (re,findex) where re is a compiled
                                      # regular expression and findex is a list
                                      # mapping regex group numbers to rules
        self.lexretext = None         # Current regular expression strings
        self.lexstatere = {}          # Dictionary mapping lexer states to master regexs
        self.lexstateretext = {}      # Dictionary mapping lexer states to regex strings
        self.lexstaterenames = {}     # Dictionary mapping lexer states to symbol names
        self.lexstate = "INITIAL"     # Current lexer state
        self.lexstatestack = []       # Stack of lexer states
        self.lexstateinfo = None      # State information
        self.lexstateignore = {}      # Dictionary of ignored characters for each state
        self.lexstateerrorf = {}      # Dictionary of error functions for each state
        self.lexreflags = 0           # Optional re compile flags
        self.lexdata = None           # Actual input data (as a string)
        self.lexpos = 0               # Current position in input text
        self.lexlen = 0               # Length of the input text
        self.lexerrorf = None         # Error rule (if any)
        self.lextokens = None         # List of valid tokens
        self.lexignore = ""           # Ignored characters
        self.lexliterals = ""         # Literal characters that can be passed through
        self.lexmodule = None         # Module
        self.lineno = 1               # Current line number
        self.lexoptimize = 0          # Optimized mode

    def clone(self,object=None):
        c = copy.copy(self)

        # If the object parameter has been supplied, it means we are attaching the
        # lexer to a new object.  In this case, we have to rebind all methods in
        # the lexstatere and lexstateerrorf tables.

        if object:
            newtab = { }
            for key, ritem in self.lexstatere.items():
                newre = []
                for cre, findex in ritem:
                     newfindex = []
                     for f in findex:
                         if not f or not f[0]:
                             newfindex.append(f)
                             continue
                         newfindex.append((getattr(object,f[0].__name__),f[1]))
                newre.append((cre,newfindex))
                newtab[key] = newre
            c.lexstatere = newtab
            c.lexstateerrorf = { }
            for key, ef in self.lexstateerrorf.items():
                c.lexstateerrorf[key] = getattr(object,ef.__name__)
            c.lexmodule = object
        return c

    # ------------------------------------------------------------
    # writetab() - Write lexer information to a table file
    # ------------------------------------------------------------
    def writetab(self,tabfile,outputdir=""):
        if isinstance(tabfile,types.ModuleType):
            return
        basetabfilename = tabfile.split(".")[-1]
        filename = os.path.join(outputdir,basetabfilename)+".py"
        tf = open(filename,"w")
        tf.write("# %s.py. This file automatically created by PLY (version %s). Don't edit!\n" % (tabfile,__version__))
        tf.write("_tabversion   = %s\n" % repr(__version__))
        tf.write("_lextokens    = %s\n" % repr(self.lextokens))
        tf.write("_lexreflags   = %s\n" % repr(self.lexreflags))
        tf.write("_lexliterals  = %s\n" % repr(self.lexliterals))
        tf.write("_lexstateinfo = %s\n" % repr(self.lexstateinfo))

        tabre = { }
        # Collect all functions in the initial state
        initial = self.lexstatere["INITIAL"]
        initialfuncs = []
        for part in initial:
            for f in part[1]:
                if f and f[0]:
                    initialfuncs.append(f)

        for key, lre in self.lexstatere.items():
             titem = []
             for i in range(len(lre)):
                  titem.append((self.lexstateretext[key][i],_funcs_to_names(lre[i][1],self.lexstaterenames[key][i])))
             tabre[key] = titem

        tf.write("_lexstatere   = %s\n" % repr(tabre))
        tf.write("_lexstateignore = %s\n" % repr(self.lexstateignore))

        taberr = { }
        for key, ef in self.lexstateerrorf.items():
             if ef:
                  taberr[key] = ef.__name__
             else:
                  taberr[key] = None
        tf.write("_lexstateerrorf = %s\n" % repr(taberr))
        tf.close()

    # ------------------------------------------------------------
    # readtab() - Read lexer information from a tab file
    # ------------------------------------------------------------
    def readtab(self,tabfile,fdict):
        if isinstance(tabfile,types.ModuleType):
            lextab = tabfile
        else:
            if sys.version_info[0] < 3:
                exec("import %s as lextab" % tabfile)
            else:
                env = { }
                exec("import %s as lextab" % tabfile, env,env)
                lextab = env['lextab']

        if getattr(lextab,"_tabversion","0.0") != __version__:
            raise ImportError("Inconsistent PLY version")

        self.lextokens      = lextab._lextokens
        self.lexreflags     = lextab._lexreflags
        self.lexliterals    = lextab._lexliterals
        self.lexstateinfo   = lextab._lexstateinfo
        self.lexstateignore = lextab._lexstateignore
        self.lexstatere     = { }
        self.lexstateretext = { }
        for key,lre in lextab._lexstatere.items():
             titem = []
             txtitem = []
             for i in range(len(lre)):
                  titem.append((re.compile(lre[i][0],lextab._lexreflags | re.VERBOSE),_names_to_funcs(lre[i][1],fdict)))
                  txtitem.append(lre[i][0])
             self.lexstatere[key] = titem
             self.lexstateretext[key] = txtitem
        self.lexstateerrorf = { }
        for key,ef in lextab._lexstateerrorf.items():
             self.lexstateerrorf[key] = fdict[ef]
        self.begin('INITIAL')

    # ------------------------------------------------------------
    # input() - Push a new string into the lexer
    # ------------------------------------------------------------
    def input(self,s):
        # Pull off the first character to see if s looks like a string
        c = s[:1]
        if not isinstance(c,StringTypes):
            raise ValueError("Expected a string")
        self.lexdata = s
        self.lexpos = 0
        self.lexlen = len(s)

    # ------------------------------------------------------------
    # begin() - Changes the lexing state
    # ------------------------------------------------------------
    def begin(self,state):
        if not state in self.lexstatere:
            raise ValueError("Undefined state")
        self.lexre = self.lexstatere[state]
        self.lexretext = self.lexstateretext[state]
        self.lexignore = self.lexstateignore.get(state,"")
        self.lexerrorf = self.lexstateerrorf.get(state,None)
        self.lexstate = state

    # ------------------------------------------------------------
    # push_state() - Changes the lexing state and saves old on stack
    # ------------------------------------------------------------
    def push_state(self,state):
        self.lexstatestack.append(self.lexstate)
        self.begin(state)

    # ------------------------------------------------------------
    # pop_state() - Restores the previous state
    # ------------------------------------------------------------
    def pop_state(self):
        self.begin(self.lexstatestack.pop())

    # ------------------------------------------------------------
    # current_state() - Returns the current lexing state
    # ------------------------------------------------------------
    def current_state(self):
        return self.lexstate

    # ------------------------------------------------------------
    # skip() - Skip ahead n characters
    # ------------------------------------------------------------
    def skip(self,n):
        self.lexpos += n

    # ------------------------------------------------------------
    # opttoken() - Return the next token from the Lexer
    #
    # Note: This function has been carefully implemented to be as fast
    # as possible.  Don't make changes unless you really know what
    # you are doing
    # ------------------------------------------------------------
    def token(self):
        # Make local copies of frequently referenced attributes
        lexpos    = self.lexpos
        lexlen    = self.lexlen
        lexignore = self.lexignore
        lexdata   = self.lexdata

        while lexpos < lexlen:
            # This code provides some short-circuit code for whitespace, tabs, and other ignored characters
            if lexdata[lexpos] in lexignore:
                lexpos += 1
                continue

            # Look for a regular expression match
            for lexre,lexindexfunc in self.lexre:
                m = lexre.match(lexdata,lexpos)
                if not m: continue

                # Create a token for return
                tok = LexToken()
                tok.value = m.group()
                tok.lineno = self.lineno
                tok.lexpos = lexpos

                i = m.lastindex
                func,tok.type = lexindexfunc[i]

                if not func:
                   # If no token type was set, it's an ignored token
                   if tok.type:
                      self.lexpos = m.end()
                      return tok
                   else:
                      lexpos = m.end()
                      break

                lexpos = m.end()

                # If token is processed by a function, call it

                tok.lexer = self      # Set additional attributes useful in token rules
                self.lexmatch = m
                self.lexpos = lexpos

                newtok = func(tok)

                # Every function must return a token, if nothing, we just move to next token
                if not newtok:
                    lexpos    = self.lexpos         # This is here in case user has updated lexpos.
                    lexignore = self.lexignore      # This is here in case there was a state change
                    break

                # Verify type of the token.  If not in the token map, raise an error
                if not self.lexoptimize:
                    if not newtok.type in self.lextokens:
                        raise LexError("%s:%d: Rule '%s' returned an unknown token type '%s'" % (
                            func_code(func).co_filename, func_code(func).co_firstlineno,
                            func.__name__, newtok.type),lexdata[lexpos:])

                return newtok
            else:
                # No match, see if in literals
                if lexdata[lexpos] in self.lexliterals:
                    tok = LexToken()
                    tok.value = lexdata[lexpos]
                    tok.lineno = self.lineno
                    tok.type = tok.value
                    tok.lexpos = lexpos
                    self.lexpos = lexpos + 1
                    return tok

                # No match. Call t_error() if defined.
                if self.lexerrorf:
                    tok = LexToken()
                    tok.value = self.lexdata[lexpos:]
                    tok.lineno = self.lineno
                    tok.type = "error"
                    tok.lexer = self
                    tok.lexpos = lexpos
                    self.lexpos = lexpos
                    newtok = self.lexerrorf(tok)
                    if lexpos == self.lexpos:
                        # Error method didn't change text position at all. This is an error.
                        raise LexError("Scanning error. Illegal character '%s'" % (lexdata[lexpos]), lexdata[lexpos:])
                    lexpos = self.lexpos
                    if not newtok: continue
                    return newtok

                self.lexpos = lexpos
                raise LexError("Illegal character '%s' at index %d" % (lexdata[lexpos],lexpos), lexdata[lexpos:])

        self.lexpos = lexpos + 1
        if self.lexdata is None:
             raise RuntimeError("No input string given with input()")
        return None

    # Iterator interface
    def __iter__(self):
        return self

    def next(self):
        t = self.token()
        if t is None:
            raise StopIteration
        return t

    __next__ = next

# -----------------------------------------------------------------------------
#                           ==== Lex Builder ===
#
# The functions and classes below are used to collect lexing information
# and build a Lexer object from it.
# -----------------------------------------------------------------------------

# -----------------------------------------------------------------------------
# get_caller_module_dict()
#
# This function returns a dictionary containing all of the symbols defined within
# a caller further down the call stack.  This is used to get the environment
# associated with the yacc() call if none was provided.
# -----------------------------------------------------------------------------

def get_caller_module_dict(levels):
    try:
        raise RuntimeError
    except RuntimeError:
        e,b,t = sys.exc_info()
        f = t.tb_frame
        while levels > 0:
            f = f.f_back                   
            levels -= 1
        ldict = f.f_globals.copy()
        if f.f_globals != f.f_locals:
            ldict.update(f.f_locals)

        return ldict

# -----------------------------------------------------------------------------
# _funcs_to_names()
#
# Given a list of regular expression functions, this converts it to a list
# suitable for output to a table file
# -----------------------------------------------------------------------------

def _funcs_to_names(funclist,namelist):
    result = []
    for f,name in zip(funclist,namelist):
         if f and f[0]:
             result.append((name, f[1]))
         else:
             result.append(f)
    return result

# -----------------------------------------------------------------------------
# _names_to_funcs()
#
# Given a list of regular expression function names, this converts it back to
# functions.
# -----------------------------------------------------------------------------

def _names_to_funcs(namelist,fdict):
     result = []
     for n in namelist:
          if n and n[0]:
              result.append((fdict[n[0]],n[1]))
          else:
              result.append(n)
     return result

# -----------------------------------------------------------------------------
# _form_master_re()
#
# This function takes a list of all of the regex components and attempts to
# form the master regular expression.  Given limitations in the Python re
# module, it may be necessary to break the master regex into separate expressions.
# -----------------------------------------------------------------------------

def _form_master_re(relist,reflags,ldict,toknames):
    if not relist: return []
    regex = "|".join(relist)
    try:
        lexre = re.compile(regex,re.VERBOSE | reflags)

        # Build the index to function map for the matching engine
        lexindexfunc = [ None ] * (max(lexre.groupindex.values())+1)
        lexindexnames = lexindexfunc[:]

        for f,i in lexre.groupindex.items():
            handle = ldict.get(f,None)
            if type(handle) in (types.FunctionType, types.MethodType):
                lexindexfunc[i] = (handle,toknames[f])
                lexindexnames[i] = f
            elif handle is not None:
                lexindexnames[i] = f
                if f.find("ignore_") > 0:
                    lexindexfunc[i] = (None,None)
                else:
                    lexindexfunc[i] = (None, toknames[f])
        
        return [(lexre,lexindexfunc)],[regex],[lexindexnames]
    except Exception:
        m = int(len(relist)/2)
        if m == 0: m = 1
        llist, lre, lnames = _form_master_re(relist[:m],reflags,ldict,toknames)
        rlist, rre, rnames = _form_master_re(relist[m:],reflags,ldict,toknames)
        return llist+rlist, lre+rre, lnames+rnames

# -----------------------------------------------------------------------------
# def _statetoken(s,names)
#
# Given a declaration name s of the form "t_" and a dictionary whose keys are
# state names, this function returns a tuple (states,tokenname) where states
# is a tuple of state names and tokenname is the name of the token.  For example,
# calling this with s = "t_foo_bar_SPAM" might return (('foo','bar'),'SPAM')
# -----------------------------------------------------------------------------

def _statetoken(s,names):
    nonstate = 1
    parts = s.split("_")
    for i in range(1,len(parts)):
         if not parts[i] in names and parts[i] != 'ANY': break
    if i > 1:
       states = tuple(parts[1:i])
    else:
       states = ('INITIAL',)

    if 'ANY' in states:
       states = tuple(names)

    tokenname = "_".join(parts[i:])
    return (states,tokenname)


# -----------------------------------------------------------------------------
# LexerReflect()
#
# This class represents information needed to build a lexer as extracted from a
# user's input file.
# -----------------------------------------------------------------------------
class LexerReflect(object):
    def __init__(self,ldict,log=None,reflags=0):
        self.ldict      = ldict
        self.error_func = None
        self.tokens     = []
        self.reflags    = reflags
        self.stateinfo  = { 'INITIAL' : 'inclusive'}
        self.files      = {}
        self.error      = 0

        if log is None:
            self.log = PlyLogger(sys.stderr)
        else:
            self.log = log

    # Get all of the basic information
    def get_all(self):
        self.get_tokens()
        self.get_literals()
        self.get_states()
        self.get_rules()
        
    # Validate all of the information
    def validate_all(self):
        self.validate_tokens()
        self.validate_literals()
        self.validate_rules()
        return self.error

    # Get the tokens map
    def get_tokens(self):
        tokens = self.ldict.get("tokens",None)
        if not tokens:
            self.log.error("No token list is defined")
            self.error = 1
            return

        if not isinstance(tokens,(list, tuple)):
            self.log.error("tokens must be a list or tuple")
            self.error = 1
            return
        
        if not tokens:
            self.log.error("tokens is empty")
            self.error = 1
            return

        self.tokens = tokens

    # Validate the tokens
    def validate_tokens(self):
        terminals = {}
        for n in self.tokens:
            if not _is_identifier.match(n):
                self.log.error("Bad token name '%s'",n)
                self.error = 1
            if n in terminals:
                self.log.warning("Token '%s' multiply defined", n)
            terminals[n] = 1

    # Get the literals specifier
    def get_literals(self):
        self.literals = self.ldict.get("literals","")

    # Validate literals
    def validate_literals(self):
        try:
            for c in self.literals:
                if not isinstance(c,StringTypes) or len(c) > 1:
                    self.log.error("Invalid literal %s. Must be a single character", repr(c))
                    self.error = 1
                    continue

        except TypeError:
            self.log.error("Invalid literals specification. literals must be a sequence of characters")
            self.error = 1

    def get_states(self):
        self.states = self.ldict.get("states",None)
        # Build statemap
        if self.states:
             if not isinstance(self.states,(tuple,list)):
                  self.log.error("states must be defined as a tuple or list")
                  self.error = 1
             else:
                  for s in self.states:
                        if not isinstance(s,tuple) or len(s) != 2:
                               self.log.error("Invalid state specifier %s. Must be a tuple (statename,'exclusive|inclusive')",repr(s))
                               self.error = 1
                               continue
                        name, statetype = s
                        if not isinstance(name,StringTypes):
                               self.log.error("State name %s must be a string", repr(name))
                               self.error = 1
                               continue
                        if not (statetype == 'inclusive' or statetype == 'exclusive'):
                               self.log.error("State type for state %s must be 'inclusive' or 'exclusive'",name)
                               self.error = 1
                               continue
                        if name in self.stateinfo:
                               self.log.error("State '%s' already defined",name)
                               self.error = 1
                               continue
                        self.stateinfo[name] = statetype

    # Get all of the symbols with a t_ prefix and sort them into various
    # categories (functions, strings, error functions, and ignore characters)

    def get_rules(self):
        tsymbols = [f for f in self.ldict if f[:2] == 't_' ]

        # Now build up a list of functions and a list of strings

        self.toknames = { }        # Mapping of symbols to token names
        self.funcsym =  { }        # Symbols defined as functions
        self.strsym =   { }        # Symbols defined as strings
        self.ignore   = { }        # Ignore strings by state
        self.errorf   = { }        # Error functions by state

        for s in self.stateinfo:
             self.funcsym[s] = []
             self.strsym[s] = []

        if len(tsymbols) == 0:
            self.log.error("No rules of the form t_rulename are defined")
            self.error = 1
            return

        for f in tsymbols:
            t = self.ldict[f]
            states, tokname = _statetoken(f,self.stateinfo)
            self.toknames[f] = tokname

            if hasattr(t,"__call__"):
                if tokname == 'error':
                    for s in states:
                        self.errorf[s] = t
                elif tokname == 'ignore':
                    line = func_code(t).co_firstlineno
                    file = func_code(t).co_filename
                    self.log.error("%s:%d: Rule '%s' must be defined as a string",file,line,t.__name__)
                    self.error = 1
                else:
                    for s in states: 
                        self.funcsym[s].append((f,t))
            elif isinstance(t, StringTypes):
                if tokname == 'ignore':
                    for s in states:
                        self.ignore[s] = t
                    if "\\" in t:
                        self.log.warning("%s contains a literal backslash '\\'",f)

                elif tokname == 'error':
                    self.log.error("Rule '%s' must be defined as a function", f)
                    self.error = 1
                else:
                    for s in states: 
                        self.strsym[s].append((f,t))
            else:
                self.log.error("%s not defined as a function or string", f)
                self.error = 1

        # Sort the functions by line number
        for f in self.funcsym.values():
            if sys.version_info[0] < 3:
                f.sort(lambda x,y: cmp(func_code(x[1]).co_firstlineno,func_code(y[1]).co_firstlineno))
            else:
                # Python 3.0
                f.sort(key=lambda x: func_code(x[1]).co_firstlineno)

        # Sort the strings by regular expression length
        for s in self.strsym.values():
            if sys.version_info[0] < 3:
                s.sort(lambda x,y: (len(x[1]) < len(y[1])) - (len(x[1]) > len(y[1])))
            else:
                # Python 3.0
                s.sort(key=lambda x: len(x[1]),reverse=True)

    # Validate all of the t_rules collected 
    def validate_rules(self):
        for state in self.stateinfo:
            # Validate all rules defined by functions

            

            for fname, f in self.funcsym[state]:
                line = func_code(f).co_firstlineno
                file = func_code(f).co_filename
                self.files[file] = 1

                tokname = self.toknames[fname]
                if isinstance(f, types.MethodType):
                    reqargs = 2
                else:
                    reqargs = 1
                nargs = func_code(f).co_argcount
                if nargs > reqargs:
                    self.log.error("%s:%d: Rule '%s' has too many arguments",file,line,f.__name__)
                    self.error = 1
                    continue

                if nargs < reqargs:
                    self.log.error("%s:%d: Rule '%s' requires an argument", file,line,f.__name__)
                    self.error = 1
                    continue

                if not f.__doc__:
                    self.log.error("%s:%d: No regular expression defined for rule '%s'",file,line,f.__name__)
                    self.error = 1
                    continue

                try:
                    c = re.compile("(?P<%s>%s)" % (fname,f.__doc__), re.VERBOSE | self.reflags)
                    if c.match(""):
                        self.log.error("%s:%d: Regular expression for rule '%s' matches empty string", file,line,f.__name__)
                        self.error = 1
                except re.error:
                    _etype, e, _etrace = sys.exc_info()
                    self.log.error("%s:%d: Invalid regular expression for rule '%s'. %s", file,line,f.__name__,e)
                    if '#' in f.__doc__:
                        self.log.error("%s:%d. Make sure '#' in rule '%s' is escaped with '\\#'",file,line, f.__name__)
                    self.error = 1

            # Validate all rules defined by strings
            for name,r in self.strsym[state]:
                tokname = self.toknames[name]
                if tokname == 'error':
                    self.log.error("Rule '%s' must be defined as a function", name)
                    self.error = 1
                    continue

                if not tokname in self.tokens and tokname.find("ignore_") < 0:
                    self.log.error("Rule '%s' defined for an unspecified token %s",name,tokname)
                    self.error = 1
                    continue

                try:
                    c = re.compile("(?P<%s>%s)" % (name,r),re.VERBOSE | self.reflags)
                    if (c.match("")):
                         self.log.error("Regular expression for rule '%s' matches empty string",name)
                         self.error = 1
                except re.error:
                    _etype, e, _etrace = sys.exc_info()
                    self.log.error("Invalid regular expression for rule '%s'. %s",name,e)
                    if '#' in r:
                         self.log.error("Make sure '#' in rule '%s' is escaped with '\\#'",name)
                    self.error = 1

            if not self.funcsym[state] and not self.strsym[state]:
                self.log.error("No rules defined for state '%s'",state)
                self.error = 1

            # Validate the error function
            efunc = self.errorf.get(state,None)
            if efunc:
                f = efunc
                line = func_code(f).co_firstlineno
                file = func_code(f).co_filename
                self.files[file] = 1

                if isinstance(f, types.MethodType):
                    reqargs = 2
                else:
                    reqargs = 1
                nargs = func_code(f).co_argcount
                if nargs > reqargs:
                    self.log.error("%s:%d: Rule '%s' has too many arguments",file,line,f.__name__)
                    self.error = 1

                if nargs < reqargs:
                    self.log.error("%s:%d: Rule '%s' requires an argument", file,line,f.__name__)
                    self.error = 1

        for f in self.files:
            self.validate_file(f)


    # -----------------------------------------------------------------------------
    # validate_file()
    #
    # This checks to see if there are duplicated t_rulename() functions or strings
    # in the parser input file.  This is done using a simple regular expression
    # match on each line in the given file.  
    # -----------------------------------------------------------------------------

    def validate_file(self,filename):
        import os.path
        base,ext = os.path.splitext(filename)
        if ext != '.py': return         # No idea what the file is. Return OK

        try:
            f = open(filename)
            lines = f.readlines()
            f.close()
        except IOError:
            return                      # Couldn't find the file.  Don't worry about it

        fre = re.compile(r'\s*def\s+(t_[a-zA-Z_0-9]*)\(')
        sre = re.compile(r'\s*(t_[a-zA-Z_0-9]*)\s*=')

        counthash = { }
        linen = 1
        for l in lines:
            m = fre.match(l)
            if not m:
                m = sre.match(l)
            if m:
                name = m.group(1)
                prev = counthash.get(name)
                if not prev:
                    counthash[name] = linen
                else:
                    self.log.error("%s:%d: Rule %s redefined. Previously defined on line %d",filename,linen,name,prev)
                    self.error = 1
            linen += 1
            
# -----------------------------------------------------------------------------
# lex(module)
#
# Build all of the regular expression rules from definitions in the supplied module
# -----------------------------------------------------------------------------
def lex(module=None,object=None,debug=0,optimize=0,lextab="lextab",reflags=0,nowarn=0,outputdir="", debuglog=None, errorlog=None):
    global lexer
    ldict = None
    stateinfo  = { 'INITIAL' : 'inclusive'}
    lexobj = Lexer()
    lexobj.lexoptimize = optimize
    global token,input

    if errorlog is None:
        errorlog = PlyLogger(sys.stderr)

    if debug:
        if debuglog is None:
            debuglog = PlyLogger(sys.stderr)

    # Get the module dictionary used for the lexer
    if object: module = object

    if module:
        _items = [(k,getattr(module,k)) for k in dir(module)]
        ldict = dict(_items)
    else:
        ldict = get_caller_module_dict(2)

    # Collect parser information from the dictionary
    linfo = LexerReflect(ldict,log=errorlog,reflags=reflags)
    linfo.get_all()
    if not optimize:
        if linfo.validate_all():
            raise SyntaxError("Can't build lexer")

    if optimize and lextab:
        try:
            lexobj.readtab(lextab,ldict)
            token = lexobj.token
            input = lexobj.input
            lexer = lexobj
            return lexobj

        except ImportError:
            pass

    # Dump some basic debugging information
    if debug:
        debuglog.info("lex: tokens   = %r", linfo.tokens)
        debuglog.info("lex: literals = %r", linfo.literals)
        debuglog.info("lex: states   = %r", linfo.stateinfo)

    # Build a dictionary of valid token names
    lexobj.lextokens = { }
    for n in linfo.tokens:
        lexobj.lextokens[n] = 1

    # Get literals specification
    if isinstance(linfo.literals,(list,tuple)):
        lexobj.lexliterals = type(linfo.literals[0])().join(linfo.literals)
    else:
        lexobj.lexliterals = linfo.literals

    # Get the stateinfo dictionary
    stateinfo = linfo.stateinfo

    regexs = { }
    # Build the master regular expressions
    for state in stateinfo:
        regex_list = []

        # Add rules defined by functions first
        for fname, f in linfo.funcsym[state]:
            line = func_code(f).co_firstlineno
            file = func_code(f).co_filename
            regex_list.append("(?P<%s>%s)" % (fname,f.__doc__))
            if debug:
                debuglog.info("lex: Adding rule %s -> '%s' (state '%s')",fname,f.__doc__, state)

        # Now add all of the simple rules
        for name,r in linfo.strsym[state]:
            regex_list.append("(?P<%s>%s)" % (name,r))
            if debug:
                debuglog.info("lex: Adding rule %s -> '%s' (state '%s')",name,r, state)

        regexs[state] = regex_list

    # Build the master regular expressions

    if debug:
        debuglog.info("lex: ==== MASTER REGEXS FOLLOW ====")

    for state in regexs:
        lexre, re_text, re_names = _form_master_re(regexs[state],reflags,ldict,linfo.toknames)
        lexobj.lexstatere[state] = lexre
        lexobj.lexstateretext[state] = re_text
        lexobj.lexstaterenames[state] = re_names
        if debug:
            for i in range(len(re_text)):
                debuglog.info("lex: state '%s' : regex[%d] = '%s'",state, i, re_text[i])

    # For inclusive states, we need to add the regular expressions from the INITIAL state
    for state,stype in stateinfo.items():
        if state != "INITIAL" and stype == 'inclusive':
             lexobj.lexstatere[state].extend(lexobj.lexstatere['INITIAL'])
             lexobj.lexstateretext[state].extend(lexobj.lexstateretext['INITIAL'])
             lexobj.lexstaterenames[state].extend(lexobj.lexstaterenames['INITIAL'])

    lexobj.lexstateinfo = stateinfo
    lexobj.lexre = lexobj.lexstatere["INITIAL"]
    lexobj.lexretext = lexobj.lexstateretext["INITIAL"]
    lexobj.lexreflags = reflags

    # Set up ignore variables
    lexobj.lexstateignore = linfo.ignore
    lexobj.lexignore = lexobj.lexstateignore.get("INITIAL","")

    # Set up error functions
    lexobj.lexstateerrorf = linfo.errorf
    lexobj.lexerrorf = linfo.errorf.get("INITIAL",None)
    if not lexobj.lexerrorf:
        errorlog.warning("No t_error rule is defined")

    # Check state information for ignore and error rules
    for s,stype in stateinfo.items():
        if stype == 'exclusive':
              if not s in linfo.errorf:
                   errorlog.warning("No error rule is defined for exclusive state '%s'", s)
              if not s in linfo.ignore and lexobj.lexignore:
                   errorlog.warning("No ignore rule is defined for exclusive state '%s'", s)
        elif stype == 'inclusive':
              if not s in linfo.errorf:
                   linfo.errorf[s] = linfo.errorf.get("INITIAL",None)
              if not s in linfo.ignore:
                   linfo.ignore[s] = linfo.ignore.get("INITIAL","")

    # Create global versions of the token() and input() functions
    token = lexobj.token
    input = lexobj.input
    lexer = lexobj

    # If in optimize mode, we write the lextab
    if lextab and optimize:
        lexobj.writetab(lextab,outputdir)

    return lexobj

# -----------------------------------------------------------------------------
# runmain()
#
# This runs the lexer as a main program
# -----------------------------------------------------------------------------

def runmain(lexer=None,data=None):
    if not data:
        try:
            filename = sys.argv[1]
            f = open(filename)
            data = f.read()
            f.close()
        except IndexError:
            sys.stdout.write("Reading from standard input (type EOF to end):\n")
            data = sys.stdin.read()

    if lexer:
        _input = lexer.input
    else:
        _input = input
    _input(data)
    if lexer:
        _token = lexer.token
    else:
        _token = token

    while 1:
        tok = _token()
        if not tok: break
        sys.stdout.write("(%s,%r,%d,%d)\n" % (tok.type, tok.value, tok.lineno,tok.lexpos))

# -----------------------------------------------------------------------------
# @TOKEN(regex)
#
# This decorator function can be used to set the regex expression on a function
# when its docstring might need to be set in an alternative way
# -----------------------------------------------------------------------------

def TOKEN(r):
    def set_doc(f):
        if hasattr(r,"__call__"):
            f.__doc__ = r.__doc__
        else:
            f.__doc__ = r
        return f
    return set_doc

# Alternative spelling of the TOKEN decorator
Token = TOKEN




#-------------------------------------------------------------------------------
# util.py
# 
# utility functions for Pyverilog
#
# Copyright (C) 2013, Shinya Takamaeda-Yamazaki
# License: Apache 2.0
#-------------------------------------------------------------------------------

import sys
import os
import copy

sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))) )

if sys.version_info[0] >= 3:
    from pyverilog.utils.scope import ScopeLabel, ScopeChain
else:
    from scope import ScopeLabel, ScopeChain

def toTermname(name):
    if isinstance(name, str): return toTermname_str(name)
    if isinstance(name, list): return toTermname_list(name)
    if isinstance(name, tuple): return toTermname_list(name)
    raise TypeError()

def toTermname_str(name):
    scopechain_list = []
    for n in name.split('.'):
        # fix me: to support "module.scope.signal[ptr]", lexical analyzer is required.
        scopechain_list.append(ScopeLabel(n, 'any'))
    return ScopeChain(scopechain_list)

def toTermname_list(name):
    scopechain_list = []
    for n in name:
        if not isinstance(n, str): raise TypeError()
        scopechain_list.append(ScopeLabel(n, 'any'))
    return ScopeChain(scopechain_list)

def getScope(termname):
    return termname[:-1]

def toFlatname(termname):
    #flatname = ''
    #for t in termname:
    #    flatname += str(t.scopename) + '__'
    #return flatname[:-2]
    return termname.tocode()

def splitScopeName(termname):
    scope = termname[:-1]
    scope_str = ''
    for s in scope:
        scope_str += s.scopename + '.'
    scope_str = scope_str[:-1]
    signame_str = termname[-1]
    return scope_str, signame_str

def isTopmodule(scope):
    if len(scope)==1: return True
    return False

def dictlistmerge(a, b):
    ret = {}
    ret.update(a)
    for bk, bv in b.items():
        if bk in ret:
            for bvv in bv:
                if bvv not in ret[bk]: ret[bk].append(bvv)
        else: ret[bk] = bv
    return ret

def maxValue(width):
    return 2 ** width - 1



import sys
if sys.version_info[0] < 3:
    pass
#    import version
#    import util
#    import verror
#    import signaltype
#    import scope
#    import op2mark
#    import inference
#    import transition



#-------------------------------------------------------------------------------
# inference.py
#
# Value inference
#
# Copyright (C) 2013, Shinya Takamaeda-Yamazaki
# License: Apache 2.0
#-------------------------------------------------------------------------------

import sys
import os

sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))) )

from pyverilog.dataflow.dataflow import *

if sys.version_info[0] >= 3:
    import pyverilog.utils.verror as verror
else:
    import verror as verror

this = sys.modules[__name__]

def infer(op, node):
    #if not isinstance(node, DFEvalValue): return None
    if not isinstance(node, DFEvalValue):
        raise verror.FormatError('Can not infer the value from non DFEvalValue object')
    val = node.value
    funcname = 'op_' + op
    opfunc = getattr(this, funcname, op_None)
    return opfunc(val)

def op_LessThan(val):
    minval = 0
    maxval = val - 1
    if maxval < 0:
        minval = None
        maxval = None
    return InferredValue(minval, maxval)
def op_GreaterThan(val):
    minval = val + 1
    maxval = None
    return InferredValue(minval, maxval)
def op_LassEq(val):
    minval = 0
    maxval = val
    return InferredValue(minval, maxval)
def op_GreaterEq(val):
    minval = val
    maxval = None
    return InferredValue(minval, maxval)
def op_Eq(val):
    minval = val
    maxval = val
    return InferredValue(minval, maxval)
def op_NotEq(val):
    minval = val
    maxval = val
    return InferredValue(minval, maxval, inv=True)
def op_Eql(val):
    minval = val
    maxval = val
    return InferredValue(minval, maxval)
def op_NotEql(val):
    minval = val
    maxval = val
    return InferredValue(minval, maxval, inv=True)
def op_None(val):
    raise verror.FormatError('Unsupported Comparator')

class InferredValue(object):
    def __init__(self, minval, maxval, inv=False):
        self.minval = minval
        self.maxval = maxval
        self.inv = inv
    def __repr__(self):
        ret = ''
        if self.inv:
            ret += '(INV '
        ret += 'min:'+str(self.minval)
        ret += ' max:'+str(self.maxval)
        if self.inv:
            ret += ')'
        return ret
    def invert(self):
        self.inv = not self.inv
